from leapp.models import RemovedPAMModules


class RemoveOldPAMModulesScannerLibrary(object):
    """
    Scan PAM configuration for modules that are not available in RHEL-8.

    PAM module pam_krb5 and pam_pkcs11 are no longer present in RHEL-8
    and must be removed from PAM configuration, otherwise it may lock out
    the system.
    """

    def __init__(self, pam):
        self.pam = pam

    def process(self):
        # PAM modules pam_pkcs11 and pam_krb5 are no longer available in
        # RHEL8. We must remove them because if they are left in PAM
        # configuration it may lock out the system.
        modules = []
        for module in ['pam_krb5', 'pam_pkcs11']:
            if self.pam.has(module):
                modules.append(module)

        return RemovedPAMModules(
            modules=modules
        )



import functools
import os

import pyudev

from leapp import reporting
from leapp.libraries.actor import storagescanner
from leapp.libraries.common.testutils import create_report_mocked, logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import (
    FstabEntry,
    LsblkEntry,
    LvdisplayEntry,
    MountEntry,
    PartitionEntry,
    PvsEntry,
    SystemdMountEntry,
    VgsEntry
)
from leapp.utils.report import is_inhibitor

CUR_DIR = os.path.dirname(os.path.abspath(__file__))


def test_get_partitions_info(monkeypatch):
    def is_file_readable_mocked(path):
        return False

    expected = [
        PartitionEntry(major='252', minor='0', blocks='41943040', name='vda'),
        PartitionEntry(major='252', minor='1', blocks='1048576', name='vda1'),
        PartitionEntry(major='252', minor='2', blocks='40893440', name='vda2'),
        PartitionEntry(major='253', minor='0', blocks='39837696', name='dm-0'),
        PartitionEntry(major='253', minor='1', blocks='1048576', name='dm-1')]
    assert expected == storagescanner._get_partitions_info(os.path.join(CUR_DIR, 'files/partitions'))

    monkeypatch.setattr(storagescanner, '_is_file_readable', is_file_readable_mocked)
    assert [] == storagescanner._get_partitions_info('unreadable_file')


def test_get_fstab_info(monkeypatch):
    expected = [
        FstabEntry(
            fs_spec='/dev/mapper/rhel_ibm--p8--kvm--03--guest--02-root',
            fs_file='/',
            fs_vfstype='xfs',
            fs_mntops='defaults',
            fs_freq='0',
            fs_passno='0'),
        FstabEntry(
            fs_spec='UUID=0a5215ef-1fb4-4b1b-8860-be4baa9e624c',
            fs_file='/boot',
            fs_vfstype='xfs',
            fs_mntops='defaults',
            fs_freq='0',
            fs_passno='1'),
        FstabEntry(
            fs_spec='UUID=acf9f525-3691-429f-96d7-3f8530227062',
            fs_file='/var',
            fs_vfstype='xfs',
            fs_mntops='defaults',
            fs_freq='0',
            fs_passno='0'),
        FstabEntry(
            fs_spec='UUID=d74186c9-21d5-4549-ae26-91ca9ed36f56',
            fs_file='/tmp',
            fs_vfstype='ext4',
            fs_mntops='defaults,nodev,nosuid,noexec',
            fs_freq='1',
            fs_passno='0'),
        FstabEntry(
            fs_spec='/dev/mapper/rhel_ibm--p8--kvm--03--guest--02-swap',
            fs_file='swap',
            fs_vfstype='swap',
            fs_mntops='defaults',
            fs_freq='0',
            fs_passno='0')]
    assert expected == storagescanner._get_fstab_info(os.path.join(CUR_DIR, 'files/fstab'))
    monkeypatch.setattr(storagescanner, '_is_file_readable', lambda dummy: False)
    assert [] == storagescanner._get_fstab_info('unreadable_file')


def test_invalid_fstab_info(monkeypatch):
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    storagescanner._get_fstab_info(os.path.join(CUR_DIR, 'files/invalid_fstab'))
    assert reporting.create_report.called == 1
    assert reporting.create_report.report_fields['severity'] == 'high'
    assert 'Problems with parsing data in /etc/fstab' in reporting.create_report.report_fields['title']
    assert is_inhibitor(reporting.create_report.report_fields)
    assert any("The fstab configuration file seems to be invalid" in msg for msg in api.current_logger.errmsg)


def test_get_mount_info(monkeypatch):
    expected = [
        MountEntry(
            name='sysfs', mount='/sys', tp='sysfs',
            options='rw,seclabel,nosuid,nodev,noexec,relatime'
        ),
        MountEntry(
            name='proc', mount='/proc', tp='proc',
            options='rw,nosuid,nodev,noexec,relatime'
        ),
        MountEntry(
            name='devtmpfs', mount='/dev', tp='devtmpfs',
            options='rw,seclabel,nosuid,size=16131092k,nr_inodes=4032773,mode=755'
        ),
        MountEntry(
            name='securityfs', mount='/sys/kernel/security', tp='securityfs',
            options='rw,nosuid,nodev,noexec,relatime'
        ),
        MountEntry(
            name='tmpfs', mount='/dev/shm', tp='tmpfs',
            options='rw,seclabel,nosuid,nodev'
        ),
        MountEntry(
            name='devpts', mount='/dev/pts', tp='devpts',
            options='rw,seclabel,nosuid,noexec,relatime,gid=5,mode=620,ptmxmode=000'
        ),
        MountEntry(
            name='tmpfs', mount='/run', tp='tmpfs',
            options='rw,seclabel,nosuid,nodev,mode=755'
        ),
        MountEntry(
            name='tmpfs', mount='/sys/fs/cgroup', tp='tmpfs',
            options='ro,seclabel,nosuid,nodev,noexec,mode=755'
        ),
        MountEntry(
            name='cgroup2', mount='/sys/fs/cgroup/unified', tp='cgroup2',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,nsdelegate'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/systemd', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,xattr,name=systemd'
        ),
        MountEntry(
            name='pstore', mount='/sys/fs/pstore', tp='pstore',
            options='rw,seclabel,nosuid,nodev,noexec,relatime'
        ),
        MountEntry(
            name='bpf', mount='/sys/fs/bpf', tp='bpf',
            options='rw,nosuid,nodev,noexec,relatime,mode=700'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/cpuset', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,cpuset'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/net_cls,net_prio', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,net_cls,net_prio'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/hugetlb', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,hugetlb'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/pids', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,pids'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/freezer', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,freezer'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/blkio', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,blkio'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/devices', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,devices'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/cpu,cpuacct', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,cpu,cpuacct'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/memory', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,memory'
        ),
        MountEntry(
            name='cgroup', mount='/sys/fs/cgroup/perf_event', tp='cgroup',
            options='rw,seclabel,nosuid,nodev,noexec,relatime,perf_event'
        ),
        MountEntry(
            name='configfs', mount='/sys/kernel/config', tp='configfs',
            options='rw,relatime'
        ),
        MountEntry(
            name='/dev/mapper/fedora-root', mount='/', tp='ext4',
            options='rw,seclabel,relatime'
        ),
        MountEntry(
            name='selinuxfs', mount='/sys/fs/selinux', tp='selinuxfs',
            options='rw,relatime'
        ),
        MountEntry(
            name='debugfs', mount='/sys/kernel/debug', tp='debugfs',
            options='rw,seclabel,relatime'
        ),
        MountEntry(
            name='hugetlbfs', mount='/dev/hugepages', tp='hugetlbfs',
            options='rw,seclabel,relatime,pagesize=2M'
        ),
        MountEntry(
            name='systemd-1', mount='/proc/sys/fs/binfmt_misc', tp='autofs',
            options='rw,relatime,fd=38,pgrp=1,timeout=0,minproto=5,maxproto=5,direct,pipe_ino=14019'
        ),
        MountEntry(
            name='mqueue', mount='/dev/mqueue', tp='mqueue',
            options='rw,seclabel,relatime'
        ),
        MountEntry(
            name='fusectl', mount='/sys/fs/fuse/connections', tp='fusectl',
            options='rw,relatime'
        ),
        MountEntry(
            name='tmpfs', mount='/tmp', tp='tmpfs',
            options='rw,seclabel,nosuid,nodev'
        ),
        MountEntry(
            name='/dev/nvme0n1p1', mount='/boot', tp='ext4',
            options='rw,seclabel,relatime'
        ),
        MountEntry(
            name='/dev/mapper/fedora-home', mount='/home', tp='ext4',
            options='rw,seclabel,relatime'
        ),
        MountEntry(
            name='sunrpc', mount='/var/lib/nfs/rpc_pipefs', tp='rpc_pipefs',
            options='rw,relatime'
        ),
        MountEntry(
            name='tmpfs', mount='/run/user/1000', tp='tmpfs',
            options='rw,seclabel,nosuid,nodev,relatime,size=3229704k,mode=700,uid=1000,gid=1000'
        ),
        MountEntry(
            name='tmpfs', mount='/run/user/42', tp='tmpfs',
            options='rw,seclabel,nosuid,nodev,relatime,size=3229704k,mode=700,uid=42,gid=42'
        ),
        MountEntry(
            name='gvfsd-fuse', mount='/run/user/1000/gvfs',
            tp='fuse.gvfsd-fuse', options='rw,nosuid,nodev,relatime,user_id=1000,group_id=1000'
        ),
        MountEntry(
            name='/dev/loop2p1', mount='/mnt/foo\\040bar', tp='iso9660',
            options='ro,nosuid,nodev,relatime,nojoliet,check=s,map=n,blocksize=2048'
        )
    ]

    assert expected == storagescanner._get_mount_info(os.path.join(CUR_DIR, 'files/mounts'))


def test_get_lsblk_info(monkeypatch):
    bytes_per_gb = 1 << 30

    def get_cmd_output_mocked(cmd, delim, expected_len):
        if cmd == ['lsblk', '-pbnr', '--output', 'NAME,MAJ:MIN,RM,SIZE,RO,TYPE,MOUNTPOINT']:
            output_lines_split_on_whitespace = [
                ['vda', '252:0', '0', str(40 * bytes_per_gb), '0', 'disk', ''],
                ['vda1', '252:1', '0', str(1 * bytes_per_gb), '0', 'part', '/boot'],
                ['vda2', '252:2', '0', str(39 * bytes_per_gb), '0', 'part', ''],
                ['rhel_ibm--p8--kvm--03--guest--02-root', '253:0', '0', str(38 * bytes_per_gb), '0', 'lvm', '/'],
                ['rhel_ibm--p8--kvm--03--guest--02-swap', '253:1', '0', str(1 * bytes_per_gb), '0', 'lvm', '[SWAP]']
            ]
            for output_line_parts in output_lines_split_on_whitespace:
                yield output_line_parts
        elif len(cmd) == 5 and cmd[:4] == ['lsblk', '-nr', '--output', 'NAME,KNAME,SIZE']:
            # We cannot have the output in a list, since the command is called per device. Therefore, we have to map
            # each device path to its output.
            output_lines_split_on_whitespace_per_device = {
                'vda': ['vda', 'vda', '40G'],
                'vda1': ['vda1', 'vda1', '1G'],
                'vda2': ['vda2', 'vda2', '39G'],
                'rhel_ibm--p8--kvm--03--guest--02-root': ['rhel_ibm--p8--kvm--03--guest--02-root', 'kname1', '38G'],
                'rhel_ibm--p8--kvm--03--guest--02-swap': ['rhel_ibm--p8--kvm--03--guest--02-swap', 'kname2', '1G']
            }
            dev_path = cmd[4]
            if dev_path not in output_lines_split_on_whitespace_per_device:
                raise ValueError('Attempting to call lsblk on an unexpected device: {}'.format(dev_path))
            yield output_lines_split_on_whitespace_per_device[dev_path]

        else:
            raise ValueError('Attempting to call unexpected command: {}'.format(cmd))

    monkeypatch.setattr(storagescanner, '_get_cmd_output', get_cmd_output_mocked)
    expected = [
        LsblkEntry(
            name='vda',
            kname='vda',
            maj_min='252:0',
            rm='0',
            size='40G',
            bsize=40 * bytes_per_gb,
            ro='0',
            tp='disk',
            mountpoint=''),
        LsblkEntry(
            name='vda1',
            kname='vda1',
            maj_min='252:1',
            rm='0',
            size='1G',
            bsize=1 * bytes_per_gb,
            ro='0',
            tp='part',
            mountpoint='/boot'),
        LsblkEntry(
            name='vda2',
            kname='vda2',
            maj_min='252:2',
            rm='0',
            size='39G',
            bsize=39 * bytes_per_gb,
            ro='0',
            tp='part',
            mountpoint=''),
        LsblkEntry(
            name='rhel_ibm--p8--kvm--03--guest--02-root',
            kname='kname1',
            maj_min='253:0',
            rm='0',
            size='38G',
            bsize=38 * bytes_per_gb,
            ro='0',
            tp='lvm',
            mountpoint='/'),
        LsblkEntry(
            name='rhel_ibm--p8--kvm--03--guest--02-swap',
            kname='kname2',
            maj_min='253:1',
            rm='0',
            size='1G',
            bsize=1 * bytes_per_gb,
            ro='0',
            tp='lvm',
            mountpoint='[SWAP]')]

    actual = storagescanner._get_lsblk_info()
    assert expected == actual


def test_get_pvs_info(monkeypatch):
    def get_cmd_output_mocked(cmd, delim, expected_len):
        return [
            ['/dev/vda2', 'rhel_ibm-p8-kvm-03-guest-02', 'lvm2', 'a--', '<39.00g', '4.00m']]

    monkeypatch.setattr(storagescanner, '_get_cmd_output', get_cmd_output_mocked)
    expected = [
        PvsEntry(
            pv='/dev/vda2',
            vg='rhel_ibm-p8-kvm-03-guest-02',
            fmt='lvm2',
            attr='a--',
            psize='<39.00g',
            pfree='4.00m')]
    assert expected == storagescanner._get_pvs_info()


def test_get_vgs_info(monkeypatch):
    def get_cmd_output_mocked(cmd, delim, expected_len):
        return [
            ['rhel_ibm-p8-kvm-03-guest-02', '1', '2', '0', 'wz--n-', '<39.00g', '4.00m']]

    monkeypatch.setattr(storagescanner, '_get_cmd_output', get_cmd_output_mocked)
    expected = [
        VgsEntry(
            vg='rhel_ibm-p8-kvm-03-guest-02',
            pv='1',
            lv='2',
            sn='0',
            attr='wz--n-',
            vsize='<39.00g',
            vfree='4.00m')]
    assert expected == storagescanner._get_vgs_info()


def test_get_lvdisplay_info(monkeypatch):
    def get_cmd_output_mocked(cmd, delim, expected_len):
        return [
            ['root', 'rhel_ibm-p8-kvm-03-guest-02', '-wi-ao----', '37.99g', '', '', '', '', '', '', '', ''],
            ['swap', 'rhel_ibm-p8-kvm-03-guest-02', '-wi-ao----', '1.00g', '', '', '', '', '', '', '', '']]

    monkeypatch.setattr(storagescanner, '_get_cmd_output', get_cmd_output_mocked)
    expected = [
        LvdisplayEntry(
            lv='root',
            vg='rhel_ibm-p8-kvm-03-guest-02',
            attr='-wi-ao----',
            lsize='37.99g',
            pool='',
            origin='',
            data='',
            meta='',
            move='',
            log='',
            cpy_sync='',
            convert=''),
        LvdisplayEntry(
            lv='swap',
            vg='rhel_ibm-p8-kvm-03-guest-02',
            attr='-wi-ao----',
            lsize='1.00g',
            pool='',
            origin='',
            data='',
            meta='',
            move='',
            log='',
            cpy_sync='',
            convert='')]
    assert expected == storagescanner._get_lvdisplay_info()


def test_get_systemd_mount_info(monkeypatch):

    class UdevDeviceMocked(object):
        def __init__(self, device_node, path, model, wwn, fs_type, label, uuid):
            self.device_node = device_node
            # Simulate udev device attributes that should be queried
            self.device_attributes = {
                'ID_PATH': path,
                'ID_MODEL': model,
                'ID_WWN': wwn,
                'ID_FS_TYPE': fs_type,
                'ID_FS_LABEL': label,
                'ID_FS_UUID': uuid,
            }

        def get(self, attribute, default=None):
            if attribute not in self.device_attributes:
                raise KeyError('Actor tried to query an udev device attribute that is not a part of the mocks.')

            if self.device_attributes[attribute] is None:
                return default

            return self.device_attributes[attribute]

    class UdevContextMocked(object):
        def __init__(self, mocked_devices):
            self.mocked_devices = mocked_devices

        def list_devices(self, **dummy_kwargs):
            return self.mocked_devices

    mocked_block_devices = [
        UdevDeviceMocked(
            device_node='/dev/dm-1',
            path=None,
            model=None,
            wwn=None,
            fs_type='ext4',
            label=None,
            uuid='bec30ca5-5403-4c23-ae6e-cb2a911bc076'),
        UdevDeviceMocked(
            device_node='/dev/dm-3',
            path=None,
            model=None,
            wwn=None,
            fs_type='ext4',
            label=None,
            uuid='d6eaf17d-e2a9-4e8d-bb54-a89c18923ea2'),
        UdevDeviceMocked(
            device_node='/dev/sda1',
            path='pci-0000:00:17.0-ata-2',
            model='LITEON_LCH-256V2S',
            wwn='0x5002303100d82b06',
            fs_type='ext4',
            label=None,
            uuid='c3890bf3-9273-4877-ad1f-68144e1eb858')]

    # Partially apply mocked_block_devices to the UdevContextMocked, so that it
    # is OK to initialize it with no arguments (same as original Context)
    monkeypatch.setattr(pyudev, 'Context', functools.partial(UdevContextMocked, mocked_block_devices))
    expected = [
        SystemdMountEntry(
            node='/dev/dm-1',
            path='n/a',
            model='n/a',
            wwn='n/a',
            fs_type='ext4',
            label='n/a',
            uuid='bec30ca5-5403-4c23-ae6e-cb2a911bc076'),
        SystemdMountEntry(
            node='/dev/dm-3',
            path='n/a',
            model='n/a',
            wwn='n/a',
            fs_type='ext4',
            label='n/a',
            uuid='d6eaf17d-e2a9-4e8d-bb54-a89c18923ea2'),
        SystemdMountEntry(
            node='/dev/sda1',
            path='pci-0000:00:17.0-ata-2',
            model='LITEON_LCH-256V2S',
            wwn='0x5002303100d82b06',
            fs_type='ext4',
            label='n/a',
            uuid='c3890bf3-9273-4877-ad1f-68144e1eb858')]
    assert expected == storagescanner._get_systemd_mount_info()



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class SpamassassinFacts(Model):
    topic = SystemInfoTopic

    spamc_ssl_argument = fields.Nullable(fields.String())
    """
    SSL version specified by the --ssl option in the spamc config file, or None
    if no value is given.
    """

    spamd_ssl_version = fields.Nullable(fields.String())
    """
    SSL version specified by the --ssl-version in the spamassassin sysconfig file,
    or None if no value is given.
    """

    service_overriden = fields.Boolean()
    """
    True if spamassassin.service is overridden, else False.
    """



import pytest

from leapp.libraries.actor.readconf import parse_config, produce_config, strip_whitespace_and_comments
from leapp.models import OpenSslConfig, OpenSslConfigBlock, OpenSslConfigPair

testdata = (
    ('key = value', 'key = value'),  # normal formatting
    ('  key = value    ', 'key = value'),  # trailing and leading whitespace
    ('  key   =   value    ', 'key   =   value'),  # whitespace in between is kept
    ('[ section ]', '[ section ]'),  # normal section formatting
    ('    [ section ]    ', '[ section ]'),  # trailing and leading whitespace
    ('[    section    ]', '[    section    ]'),  # whitespace in braces is kept
    ('# comment', ''),  # only comment
    ('    # comment   ', ''),  # only comment with whitespace
    ('key = value# comment', 'key = value'),  # key value with comment
    ('key = value     # comment', 'key = value'),  # key value with comment and more whitespace
    ('[ section ]# comment', '[ section ]'),  # the section with comment
    ('[ section ]     # comment', '[ section ]'),  # the section with comment and more whitespace
)


@pytest.mark.parametrize('line,expected_result', testdata)
def test_strip_whitespace_and_comments(line, expected_result):
    result = strip_whitespace_and_comments(line)
    assert result == expected_result


def test_parse_config():
    config = [
        "# comment from file",
        "",  # empty line
        "   ",  # whitespace line
        "HOME                    = .",  # ignored before the start of block
        "openssl_conf = default_modules",  # the start key
        "# key = value in comment",  # ignored
        "HOME                    = .",  # ignored again
        "[ default_modules ]",  # first block
        "ssl_conf = ssl_module",
        "[ ssl_module ]",
        "system_default = crypto_policy",
        "[ crypto_policy ]",
        "# key = value in comment",  # ignored
        ".include = /etc/crypto-policies/back-ends/opensslcnf.config",
    ]

    output = parse_config(config)
    assert isinstance(output, OpenSslConfig)
    assert output.openssl_conf == "default_modules"
    assert len(output.blocks) == 3
    assert output.blocks[0].name == "default_modules"
    assert len(output.blocks[0].pairs) == 1
    assert output.blocks[0].pairs[0].key == "ssl_conf"
    assert output.blocks[0].pairs[0].value == "ssl_module"
    assert output.blocks[1].name == "ssl_module"
    assert len(output.blocks[1].pairs) == 1
    assert output.blocks[1].pairs[0].key == "system_default"
    assert output.blocks[1].pairs[0].value == "crypto_policy"
    assert output.blocks[2].name == "crypto_policy"
    assert len(output.blocks[2].pairs) == 1
    assert output.blocks[2].pairs[0].key == ".include"
    assert output.blocks[2].pairs[0].value == "/etc/crypto-policies/back-ends/opensslcnf.config"


def test_parse_config_empty():
    output = parse_config([])
    assert isinstance(output, OpenSslConfig)
    assert not output.openssl_conf
    assert len(output.blocks) == 0


def test_parse_config_bare_include():
    config = [
        "[ crypto_policy ]",
        ".include    /etc/crypto-policies/back-ends/opensslcnf.config",
    ]
    output = parse_config(config)
    assert isinstance(output, OpenSslConfig)
    assert not output.openssl_conf
    assert len(output.blocks) == 1
    assert output.blocks[0].name == "crypto_policy"
    assert len(output.blocks[0].pairs) == 1
    assert output.blocks[0].pairs[0].key == ".include"
    assert output.blocks[0].pairs[0].value == "/etc/crypto-policies/back-ends/opensslcnf.config"


def test_produce_config():
    output = []

    def fake_producer(*args):
        output.extend(args)

    config = OpenSslConfig(
        openssl_conf="default_modules",
        blocks=[
            OpenSslConfigBlock(
                name="default_modules",
                pairs=[
                    OpenSslConfigPair(
                        key="ssl_conf",
                        value="ssl_module"
                    )
                ]
            ),
            OpenSslConfigBlock(
                name="ssl_module",
                pairs=[
                    OpenSslConfigPair(
                        key="system_default",
                        value="crypto_policy"
                    )
                ]
            ),
            OpenSslConfigBlock(
                name="crypto_policy",
                pairs=[
                    OpenSslConfigPair(
                        key=".include",
                        value="/etc/crypto-policies/back-ends/opensslcnf.config"
                    )
                ]
            )
        ]
    )

    produce_config(fake_producer, config)
    assert len(output) == 1
    cfg = output[0]
    assert cfg.openssl_conf == "default_modules"
    assert len(cfg.blocks) == 3
    assert cfg.blocks[0].name == "default_modules"
    assert len(cfg.blocks[0].pairs) == 1
    assert cfg.blocks[0].pairs[0].key == "ssl_conf"
    assert cfg.blocks[0].pairs[0].value == "ssl_module"
    assert cfg.blocks[1].name == "ssl_module"
    assert len(cfg.blocks[1].pairs) == 1
    assert cfg.blocks[1].pairs[0].key == "system_default"
    assert cfg.blocks[1].pairs[0].value == "crypto_policy"
    assert cfg.blocks[2].name == "crypto_policy"
    assert len(cfg.blocks[2].pairs) == 1
    assert cfg.blocks[2].pairs[0].key == ".include"
    assert cfg.blocks[2].pairs[0].value == "/etc/crypto-policies/back-ends/opensslcnf.config"


def test_actor_execution(current_actor_context):
    current_actor_context.run()
    assert current_actor_context.consume(OpenSslConfig)



import mock

from leapp.libraries.common import rpms
from leapp.libraries.common.config import mock_configs
from leapp.models import (
    DistributionSignedRPM,
    fields,
    InstalledRedHatSignedRPM,
    InstalledRPM,
    InstalledUnsignedRPM,
    IPUConfig,
    Model,
    OSRelease,
    RPM
)

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'


class MockObject(Model):
    topic = RPM.topic
    value = fields.Integer(default=42)
    plan = fields.Nullable(fields.String())


class MockModel(Model):
    topic = RPM.topic
    list_field = fields.List(fields.Model(MockObject), default=[])
    list_field_nullable = fields.Nullable(fields.List(fields.String()))
    int_field = fields.Integer(default=42)


def test_no_installed_rpms(current_actor_context):
    current_actor_context.run(config_model=mock_configs.CONFIG)
    assert current_actor_context.consume(DistributionSignedRPM)
    assert current_actor_context.consume(InstalledRedHatSignedRPM)
    assert current_actor_context.consume(InstalledUnsignedRPM)


def test_actor_execution_with_signed_unsigned_data(current_actor_context):
    installed_rpm = [
        RPM(name='sample01', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
        RPM(name='sample02', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample03', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 5326810137017186'),
        RPM(name='sample04', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample05', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 938a80caf21541eb'),
        RPM(name='sample06', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample07', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID fd372689897da07a'),
        RPM(name='sample08', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample09', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 45689c882fa658e0')]

    current_actor_context.feed(InstalledRPM(items=installed_rpm))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    assert current_actor_context.consume(DistributionSignedRPM)
    assert len(current_actor_context.consume(DistributionSignedRPM)[0].items) == 5
    assert current_actor_context.consume(InstalledRedHatSignedRPM)
    assert len(current_actor_context.consume(InstalledRedHatSignedRPM)[0].items) == 5
    assert current_actor_context.consume(InstalledUnsignedRPM)
    assert len(current_actor_context.consume(InstalledUnsignedRPM)[0].items) == 4


def test_actor_execution_with_signed_unsigned_data_centos(current_actor_context):
    CENTOS_PACKAGER = 'CentOS BuildSystem <http://bugs.centos.org>'
    config = mock_configs.CONFIG

    config.os_release = OSRelease(
        release_id='centos',
        name='CentOS Linux',
        pretty_name='CentOS Linux 7 (Core)',
        version='7 (Core)',
        version_id='7'
    )

    installed_rpm = [
        RPM(name='sample01', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 24c6a8a7f4a80eb5'),
        RPM(name='sample02', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample03', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 05b555b38483c65d'),
        RPM(name='sample04', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample05', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 4eb84e71f2ee9d55'),
        RPM(name='sample06', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample07', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID fd372689897da07a'),
        RPM(name='sample08', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample09', version='0.1', release='1.sm01', epoch='1', packager=CENTOS_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 45689c882fa658e0')]

    current_actor_context.feed(InstalledRPM(items=installed_rpm))
    current_actor_context.run(config_model=config)
    assert current_actor_context.consume(DistributionSignedRPM)
    assert len(current_actor_context.consume(DistributionSignedRPM)[0].items) == 3
    assert current_actor_context.consume(InstalledRedHatSignedRPM)
    assert not current_actor_context.consume(InstalledRedHatSignedRPM)[0].items
    assert current_actor_context.consume(InstalledUnsignedRPM)
    assert len(current_actor_context.consume(InstalledUnsignedRPM)[0].items) == 6


def test_actor_execution_with_unknown_distro(current_actor_context):
    config = mock_configs.CONFIG

    config.os_release = OSRelease(
        release_id='myos',
        name='MyOS Linux',
        pretty_name='MyOS Linux 7 (Core)',
        version='7 (Core)',
        version_id='7'
    )

    current_actor_context.feed(InstalledRPM(items=[]))
    current_actor_context.run(config_model=config)
    assert not current_actor_context.consume(DistributionSignedRPM)
    assert not current_actor_context.consume(InstalledRedHatSignedRPM)
    assert not current_actor_context.consume(InstalledUnsignedRPM)


def test_all_rpms_signed(current_actor_context):
    installed_rpm = [
        RPM(name='sample01', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
        RPM(name='sample02', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
        RPM(name='sample03', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 5326810137017186'),
        RPM(name='sample04', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X')
    ]

    current_actor_context.feed(InstalledRPM(items=installed_rpm))
    current_actor_context.run(config_model=mock_configs.CONFIG_ALL_SIGNED)
    assert current_actor_context.consume(DistributionSignedRPM)
    assert len(current_actor_context.consume(DistributionSignedRPM)[0].items) == 4
    assert current_actor_context.consume(InstalledRedHatSignedRPM)
    assert len(current_actor_context.consume(InstalledRedHatSignedRPM)[0].items) == 4
    assert not current_actor_context.consume(InstalledUnsignedRPM)[0].items


def test_katello_pkg_goes_to_signed(current_actor_context):
    installed_rpm = [
        RPM(name='katello-ca-consumer-vm-098.example.com',
            version='1.0',
            release='1',
            epoch='0',
            packager='None',
            arch='noarch',
            pgpsig=''),
    ]

    current_actor_context.feed(InstalledRPM(items=installed_rpm))
    current_actor_context.run(config_model=mock_configs.CONFIG_ALL_SIGNED)
    assert current_actor_context.consume(DistributionSignedRPM)
    assert len(current_actor_context.consume(DistributionSignedRPM)[0].items) == 1
    assert current_actor_context.consume(InstalledRedHatSignedRPM)
    assert len(current_actor_context.consume(InstalledRedHatSignedRPM)[0].items) == 1
    assert not current_actor_context.consume(InstalledUnsignedRPM)[0].items


def test_gpg_pubkey_pkg(current_actor_context):
    installed_rpm = [
        RPM(name='gpg-pubkey', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID aa17105e03152d37'),
        RPM(name='gpg-pubkey', version='0.1', release='1.sm01', epoch='1', packager='Tester', arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 9ea903b1361e896b'),
    ]

    current_actor_context.feed(InstalledRPM(items=installed_rpm))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    assert current_actor_context.consume(DistributionSignedRPM)
    assert len(current_actor_context.consume(DistributionSignedRPM)[0].items) == 2
    assert current_actor_context.consume(InstalledRedHatSignedRPM)
    assert len(current_actor_context.consume(InstalledRedHatSignedRPM)[0].items) == 2
    assert current_actor_context.consume(InstalledUnsignedRPM)
    assert not current_actor_context.consume(InstalledUnsignedRPM)[0].items


def test_create_lookup():
    # NOTE(ivasilev) Ideally should be tested separately from the actor, but since library
    # testing functionality is not yet implemented in leapp-repository the tests will reside here.
    model = MockModel(list_field=[MockObject(value=42, plan="A"),
                                  MockObject(value=-42, plan="B"),
                                  MockObject(value=9999)])
    # plain non-empty list
    keys = ('value', )
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'list_field', keys=keys)
        assert {(42, ), (-42, ), (9999, )} == lookup
    # plain list, multiple keys
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'list_field', keys=('value', 'plan'))
        assert {(42, 'A'), (-42, 'B'), (9999, None)} == lookup
    # empty list
    model.list_field = []
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'list_field', keys=keys)
        assert set() == lookup
    # nullable list without default
    assert model.list_field_nullable is None
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'list_field_nullable', keys=keys)
        assert set() == lookup
    # improper usage: lookup from non iterable field
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'int_field', keys=keys)
        assert set() == lookup
    # improper usage: lookup from iterable but bad attribute
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'list_field', keys=('nosuchattr',))
        assert set() == lookup
    # improper usage: lookup from iterable, multiple keys bad 1 bad
    with mock.patch('leapp.libraries.stdlib.api.consume', return_value=(model,)):
        lookup = rpms.create_lookup(MockModel, 'list_field', keys=('value', 'nosuchattr'))
        assert set() == lookup


def test_has_package(current_actor_context):
    installed_rpm = [
        RPM(name='sample01', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='RSA/SHA256, Mon 01 Jan 1970 00:00:00 AM -03, Key ID 199e2f91fd431d51'),
        RPM(name='sample02', version='0.1', release='1.sm01', epoch='1', packager=RH_PACKAGER, arch='noarch',
            pgpsig='SOME_OTHER_SIG_X'),
    ]

    current_actor_context.feed(InstalledRPM(items=installed_rpm))
    current_actor_context.run(config_model=mock_configs.CONFIG)
    assert rpms.has_package(DistributionSignedRPM, 'sample01', context=current_actor_context)
    assert not rpms.has_package(DistributionSignedRPM, 'nosuchpackage', context=current_actor_context)
    assert rpms.has_package(InstalledRedHatSignedRPM, 'sample01', context=current_actor_context)
    assert not rpms.has_package(InstalledRedHatSignedRPM, 'nosuchpackage', context=current_actor_context)
    assert rpms.has_package(InstalledUnsignedRPM, 'sample02', context=current_actor_context)
    assert not rpms.has_package(InstalledUnsignedRPM, 'nosuchpackage', context=current_actor_context)



from leapp.actors import Actor
from leapp.libraries.actor.grubenvtofile import grubenv_to_file
from leapp.models import HybridImage
from leapp.tags import FinalizationPhaseTag, IPUWorkflowTag


class GrubenvToFile(Actor):
    """
    Convert "grubenv" symlink to a regular file on Azure hybrid images using BIOS.

    Azure images provided by Red Hat aim for hybrid (BIOS/EFI) functionality,
    however, currently GRUB is not able to see the "grubenv" file if it is a symlink
    to a different partition (default on EFI with grub2-efi pkg installed) and
    fails on BIOS systems. This actor converts the symlink to the normal file
    with the content of grubenv on the EFI partition in case the system is using BIOS
    and running on the Azure cloud. This action is reported in the preupgrade phase.
    """

    name = 'grubenvtofile'
    consumes = (HybridImage,)
    produces = ()
    tags = (FinalizationPhaseTag, IPUWorkflowTag)

    def process(self):
        grubenv_msg = next(self.consume(HybridImage), None)

        if grubenv_msg and grubenv_msg.detected:
            grubenv_to_file()



from leapp.models import FirewalldFacts
from leapp.reporting import Report
from leapp.utils.report import is_inhibitor


def test_actor_execution(current_actor_context):
    current_actor_context.feed(
        FirewalldFacts(firewall_config_command='',
                       ebtablesTablesInUse=['broute'],
                       ipsetTypesInUse=['hash:net,port']))
    current_actor_context.run()
    report_fileds = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fileds)



import re

import pyudev

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api
from leapp.models import KernelCmdlineArg, PersistentNetNamesFacts


def is_biosdevname_disabled():
    with open('/proc/cmdline') as cmdline:
        if 'biosdevname=0' in cmdline.read():
            return True

    return False


def is_vendor_dell():
    context = pyudev.Context()

    # There should be only one dmi/id device
    for dev in pyudev.Enumerator(context).match_subsystem('dmi').match_sys_name('id'):
        vendor = dev.attributes.get('sys_vendor')
        return re.search('Dell.*', str(vendor)) is not None
    return False


def all_interfaces_biosdevname(interfaces):
    # Biosdevname supports two naming schemes
    emx = re.compile('em[0-9]+')
    pxpy = re.compile('p[0-9]+p[0-9]+')

    for i in interfaces:
        if emx.match(i.name) is None and pxpy.match(i.name) is None:
            return False
    return True


def enable_biosdevname():
    api.current_logger().info(
        "Biosdevname naming scheme in use, explicitly enabling biosdevname on the target RHEL system"
    )
    api.produce(KernelCmdlineArg(**{'key': 'biosdevname', 'value': '1'}))


def check_biosdevname():
    if is_biosdevname_disabled():
        return

    net_names_facts_messages = api.consume(PersistentNetNamesFacts)
    net_names_facts = next(net_names_facts_messages, None)
    if not net_names_facts:
        raise StopActorExecutionError(
            'Could not read interfaces names',
            details={'details': 'No PersistentNetNamesFacts message found.'},
        )

    if is_vendor_dell() and all_interfaces_biosdevname(net_names_facts.interfaces):
        enable_biosdevname()



import argparse
import ast
import os
import sys


def is_direct_actor_def(ast_node):
    if not isinstance(ast_node, ast.ClassDef):
        return False

    direcly_named_bases = (base for base in ast_node.bases if isinstance(base, ast.Name))
    for class_base in direcly_named_bases:
        # We are looking for direct name 'Actor'
        if class_base.id == 'Actor':
            return True

    return False


def extract_actor_name_from_def(actor_class_def):
    assignment_value_class = ast.Str if sys.version_info < (3,8) else ast.Constant
    assignment_value_attrib = 's' if sys.version_info < (3,8) else 'value'

    actor_name = None
    class_level_assignments = (child for child in actor_class_def.body if isinstance(child, ast.Assign))
    # Search for class-level assignment specifying actor's name: `name = 'name'`
    for child in class_level_assignments:
        assignment = child
        for target in assignment.targets:
            assignment_adds_name_attrib = isinstance(target, ast.Name) and target.id == 'name'
            assignment_uses_a_constant_string = isinstance(assignment.value, assignment_value_class)
            if assignment_adds_name_attrib and assignment_uses_a_constant_string:
                rhs = assignment.value  # <lhs> = <rhs>
                actor_name = getattr(rhs, assignment_value_attrib)
                break
        if actor_name is not None:
            break
    return actor_name


def get_actor_names(actor_path):
    with open(actor_path) as actor_file:
        try:
            actor_def = ast.parse(actor_file.read())
        except SyntaxError:
            error = ('Failed to parse {0}. The actor might contain syntax errors, or perhaps it '
                     'is written with Python3-specific syntax?\n')
            sys.stderr.write(error.format(actor_path))
            return []
        actor_defs = [ast_node for ast_node in actor_def.body if is_direct_actor_def(ast_node)]
        actors = [extract_actor_name_from_def(actor_def) for actor_def in actor_defs]
    return actors


def make_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('actor_names', nargs='+',
                        help='Actor names (the name attribute of the actor class) to look for.')
    parser.add_argument('-C', '--change-dir', dest='cwd',
                        help='Path in which the actors will be looked for.', default='.')
    return parser


if __name__ == '__main__':
    parser = make_parser()
    args = parser.parse_args()
    cwd = os.path.abspath(args.cwd)
    actor_names_to_search_for = set(args.actor_names)

    actor_paths = []
    for directory, dummy_subdirs, dir_files in os.walk(cwd):
        for actor_path in dir_files:
            actor_path = os.path.join(directory, actor_path)
            if os.path.basename(actor_path) != 'actor.py':
                continue

            defined_actor_names = set(get_actor_names(actor_path))
            if defined_actor_names.intersection(actor_names_to_search_for):
                actor_module_path = directory
                actor_paths.append(actor_module_path)
    print('\n'.join(actor_paths))



from leapp import reporting
from leapp.libraries.common.gpg import is_nogpgcheck_set
from leapp.libraries.common.rpms import get_installed_rpms
from leapp.libraries.stdlib import api
from leapp.models import TrustedGpgKeys

FMT_LIST_SEPARATOR = '\n    - '


def _get_installed_fps_tuple():
    """
    Return list of tuples (fingerprint, packager).
    """
    installed_fps_tuple = []
    rpms = get_installed_rpms()
    for rpm in rpms:
        rpm = rpm.strip()
        if not rpm:
            continue
        try:
            # NOTE: pgpsig is (none) for 'gpg-pubkey' entries
            name, version, dummy_release, dummy_epoch, packager, dummy_arch, dummy_pgpsig = rpm.split('|')
        except ValueError as e:
            # NOTE: it's seatbelt, but if it happens, seeing loong list of errors
            # will let us know earlier that we missed something really
            api.current_logger().error('Cannot perform the check of installed GPG keys after the upgrade.')
            api.current_logger().error('Cannot parse rpm output: {}'.format(e))
            continue
        if name != 'gpg-pubkey':
            continue
        installed_fps_tuple.append((version, packager))
    return installed_fps_tuple


def _report_cannot_check_keys(installed_fps):
    # NOTE: in this case, it's expected there will be always some GPG keys present
    summary = (
        'Cannot perform the check of GPG keys installed in the RPM DB'
        ' due to missing facts (TrustedGpgKeys) supposed to be generated'
        ' in the start of the upgrade process on the original system.'
        ' Unexpected unexpected installed GPG keys could be e.g. a mark of'
        ' a malicious attempt to hijack the upgrade process.'
        ' The list of all GPG keys in RPM DB:{sep}{key_list}'
        .format(
            sep=FMT_LIST_SEPARATOR,
            key_list=FMT_LIST_SEPARATOR.join(installed_fps)
        )
    )
    hint = (
        'Verify the installed GPG keys are expected.'
    )
    groups = [
        reporting.Groups.POST,
        reporting.Groups.REPOSITORY,
        reporting.Groups.SECURITY
    ]
    reporting.create_report([
        reporting.Title('Cannot perform the check of installed GPG keys after the upgrade.'),
        reporting.Summary(summary),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups(groups),
        reporting.Remediation(hint=hint),
    ])


def _report_unexpected_keys(unexpected_fps):
    summary = (
        'The system contains unexpected GPG keys after upgrade.'
        ' This can be caused e.g. by a manual intervention'
        ' or by malicious attempt to hijack the upgrade process.'
        ' The unexpected keys are the following:'
        ' {sep}{key_list}'
        .format(
            sep=FMT_LIST_SEPARATOR,
            key_list=FMT_LIST_SEPARATOR.join(unexpected_fps)
        )
    )
    hint = (
        'Verify the installed GPG keys are expected.'
    )
    groups = [
        reporting.Groups.POST,
        reporting.Groups.REPOSITORY,
        reporting.Groups.SECURITY
    ]
    reporting.create_report([
        reporting.Title('Detected unexpected GPG keys after the upgrade.'),
        reporting.Summary(summary),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups(groups),
        reporting.Remediation(hint=hint),
    ])


def process():
    """
    Verify the system does not have any unexpected gpg keys installed

    If the --no-gpgcheck option is used, this is skipped as we can not
    guarantee that what was installed came from trusted source
    """

    if is_nogpgcheck_set():
        api.current_logger().warning('The --nogpgcheck option is used: Skipping the check of installed GPG keys.')
        return

    installed_fps_tuple = _get_installed_fps_tuple()

    try:
        trusted_gpg_keys = next(api.consume(TrustedGpgKeys))
    except StopIteration:
        # unexpected (bug) situation; keeping as seatbelt for the security aspect
        installed_fps = ['{fp}: {packager}'.format(fp=fp, packager=packager) for fp, packager in installed_fps_tuple]
        _report_cannot_check_keys(installed_fps)
        return

    trusted_fps = [key.fingerprint for key in trusted_gpg_keys.items]
    unexpected_fps = []
    for fp, packager in installed_fps_tuple:
        if fp not in trusted_fps:
            unexpected_fps.append('{fp}: {packager}'.format(fp=fp, packager=packager))

    if unexpected_fps:
        _report_unexpected_keys(unexpected_fps)



from leapp import reporting
from leapp.libraries.stdlib import api, run

related = [
    reporting.RelatedResource('package', 'ntpd'),
    reporting.RelatedResource('package', 'chrony'),
    reporting.RelatedResource('file', '/etc/chrony.conf'),
]


def is_config_default():
    """Check if the chrony config file was not modified since installation."""
    try:
        result = run(['rpm', '-V', '--nomtime', 'chrony'], checked=False)
        return '/etc/chrony.conf' not in result['stdout']
    except OSError as e:
        api.current_logger().warning("rpm verification failed: %s", str(e))
        return True


def check_chrony(chrony_installed):
    """Report potential issues in chrony configuration."""
    if not chrony_installed:
        api.current_logger().info('chrony package is not installed')
        return

    if is_config_default():
        reporting.create_report([
            reporting.Title('chrony using default configuration'),
            reporting.Summary(
                'default chrony configuration in RHEL8 uses leapsectz directive, which cannot be used with '
                'leap smearing NTP servers, and uses a single pool directive instead of four server directives'
            ),
            reporting.Severity(reporting.Severity.MEDIUM),
            reporting.Groups([
                    reporting.Groups.SERVICES,
                    reporting.Groups.TIME_MANAGEMENT
            ])
        ] + related)

    else:
        reporting.create_report([
            reporting.Title('chrony using non-default configuration'),
            reporting.Summary('chrony behavior will not change in RHEL8'),
            reporting.Severity(reporting.Severity.LOW),
            reporting.Groups([
                    reporting.Groups.SERVICES,
                    reporting.Groups.TIME_MANAGEMENT
            ])
        ] + related)



from leapp import reporting
from leapp.libraries.common.config.version import get_target_major_version
from leapp.libraries.stdlib import api
from leapp.models import KernelCmdlineArg, SELinuxFacts, SelinuxPermissiveDecision, SelinuxRelabelDecision

DOC_URL = 'https://red.ht/rhel9-disabling-selinux'


def process():
    facts = next(api.consume(SELinuxFacts), None)
    if not facts:
        return

    enabled = facts.enabled
    conf_status = facts.static_mode

    if conf_status == 'disabled':
        if get_target_major_version() == '9':
            api.produce(KernelCmdlineArg(key='selinux', value='0'))
            reporting.create_report([
                reporting.Title('LEAPP detected SELinux disabled in "/etc/selinux/config"'),
                reporting.Summary(
                    'On RHEL 9, disabling SELinux in "/etc/selinux/config" is no longer possible. '
                    'This way, the system starts with SELinux enabled but with no policy loaded. LEAPP '
                    'will automatically disable SELinux using "SELINUX=0" kernel command line parameter. '
                    'However, Red Hat strongly recommends to have SELinux enabled'
                ),
                reporting.Severity(reporting.Severity.INFO),
                reporting.Groups([reporting.Groups.SELINUX]),
                reporting.RelatedResource('file', '/etc/selinux/config'),
                reporting.ExternalLink(url=DOC_URL, title='Disabling SELinux'),
            ])

        if enabled:
            reporting.create_report([
                reporting.Title('SElinux should be disabled based on the configuration file but it is enabled'),
                reporting.Summary(
                    'This message is to inform user about non-standard SElinux configuration. Please check '
                    '"/etc/selinux/config" to see whether the configuration is set as expected.'
                ),
                reporting.Severity(reporting.Severity.LOW),
                reporting.Groups([reporting.Groups.SELINUX, reporting.Groups.SECURITY])
            ])
        reporting.create_report([
            reporting.Title('SElinux disabled'),
            reporting.Summary('SElinux disabled, continuing...'),
            reporting.Groups([reporting.Groups.SELINUX, reporting.Groups.SECURITY])
        ])
        return

    if conf_status in ('enforcing', 'permissive'):
        api.produce(SelinuxRelabelDecision(set_relabel=True))
        reporting.create_report([
            reporting.Title('SElinux relabeling will be scheduled'),
            reporting.Summary('SElinux relabeling will be scheduled as the status is permissive/enforcing.'),
            reporting.Severity(reporting.Severity.INFO),
            reporting.Groups([reporting.Groups.SELINUX, reporting.Groups.SECURITY])
        ])

    if conf_status == 'enforcing':
        api.produce(SelinuxPermissiveDecision(
            set_permissive=True))
        reporting.create_report([
            reporting.Title('SElinux will be set to permissive mode'),
            reporting.Summary(
                'SElinux will be set to permissive mode. Current mode: enforcing. This action is '
                'required by the upgrade process to make sure the upgraded system can boot without '
                'beinig blocked by SElinux rules.'
            ),
            reporting.Severity(reporting.Severity.LOW),
            reporting.Remediation(hint=(
                'Make sure there are no SElinux related warnings after the upgrade and enable SElinux '
                'manually afterwards. Notice: You can ignore the "/root/tmp_leapp_py3" SElinux warnings.'
                )
            ),
            reporting.Groups([reporting.Groups.SELINUX, reporting.Groups.SECURITY])
        ])



from collections import namedtuple

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api, CalledProcessError, run

KernelPkgInfo = namedtuple('KernelPkgInfo', ('name', 'version', 'release', 'arch', 'nevra'))


KERNEL_UNAME_R_PROVIDES = ['kernel-uname-r', 'kernel-rt-uname-r']


class KernelType(object):
    ORDINARY = 'ordinary'
    REALTIME = 'realtime'


def determine_kernel_type_from_uname(rhel_version, kernel_uname_r):
    """
    Determine kernel type from given kernel release (uname-r).

    :param str rhel_version: Version of RHEL for which the kernel with the uname-r is targeted.
    :param str kernel_uname_r: Kernel release (uname-r)
    :returns: Kernel type based on a given uname_r
    :rtype: KernelType
    """
    version_fragments = rhel_version.split('.')
    major_ver = version_fragments[0]
    minor_ver = version_fragments[1] if len(version_fragments) > 1 else '0'
    rhel_version = (major_ver, minor_ver)

    if rhel_version <= ('9', '2'):
        uname_r_infixes = {
            '.rt': KernelType.REALTIME
        }
        for infix, kernel_type in uname_r_infixes.items():
            if infix in kernel_uname_r:
                return kernel_type
    else:
        uname_r_suffixes = {
            '+rt': KernelType.REALTIME
        }

        for suffix, kernel_type in uname_r_suffixes.items():
            if kernel_uname_r.endswith(suffix):
                return kernel_type

    return KernelType.ORDINARY


def get_uname_r_provided_by_kernel_pkg(kernel_pkg_nevra):
    """
    Get kernel release (uname-r) provided by a given kernel package.

    Calls the ``rpm`` command internally and might raise CalledProcessError if the rpm query fails.

    :param str kernel_pkg_nevra: NEVRA of an installed kernel package
    :returns: uname-r provided by the given package
    :rtype: str
    """
    provides = run(['rpm', '-q', '--provides', kernel_pkg_nevra],
                   split=True,
                   callback_raw=lambda fd, value: None,
                   callback_linebuffered=lambda fd, value: None)['stdout']
    for provide_line in provides:
        if '=' not in provide_line:
            continue
        provide, value = provide_line.split('=', 1)
        provide = provide.strip()
        if provide in KERNEL_UNAME_R_PROVIDES:
            return value.strip()
    return ''


def get_kernel_pkg_info(kernel_pkg_nevra):
    """
    Query the RPM database for information about the given kernel package.

    Calls the ``rpm`` command internally and might raise CalledProcessError if the rpm query fails.

    :param str kernel_pkg_nevra: NEVRA of an installed kernel package
    :returns: Information about the given kernel package
    :rtype: KernelPkgInfo
    """
    query_format = '%{NAME}|%{VERSION}|%{RELEASE}|%{ARCH}|'
    pkg_info = run(['rpm', '-q', '--queryformat', query_format, kernel_pkg_nevra])['stdout'].strip().split('|')
    return KernelPkgInfo(name=pkg_info[0], version=pkg_info[1], release=pkg_info[2], arch=pkg_info[3],
                         nevra=kernel_pkg_nevra)


def get_kernel_pkg_info_for_uname_r(uname_r):
    """
    Identify the kernel package providing a kernel with the given kernel release (uname-r).

    Raises ``StopActorExecutionError`` if no package provides given uname_r or if the internal rpm query fails.
    :param str uname_r: NEVRA of an installed kernel package
    :returns: Information about the kernel package providing given uname_r
    :rtype: KernelPkgInfo
    """
    kernel_pkg_nevras = []
    for kernel_uname_r_provide in KERNEL_UNAME_R_PROVIDES:
        try:
            kernel_pkg_nevras += run(['rpm', '-q', '--whatprovides', kernel_uname_r_provide], split=True)['stdout']
        except CalledProcessError:  # There is nothing providing a particular provide, e.g, kernel-rt-uname-r
            continue  # Nothing bad happened, continue

    kernel_pkg_nevras = set(kernel_pkg_nevras)

    for kernel_pkg_nevra in kernel_pkg_nevras:
        provided_uname = get_uname_r_provided_by_kernel_pkg(kernel_pkg_nevra)  # We know all packages provide a uname
        if not provided_uname:
            api.current_logger().warning('Failed to obtain uname-r provided by %s', kernel_pkg_nevra)
        if provided_uname == uname_r:
            return get_kernel_pkg_info(kernel_pkg_nevra)

    raise StopActorExecutionError(message='Unable to obtain kernel information of the booted kernel: no package is '
                                          'providing the booted kernel release returned by uname.')



import base64
import io
import os
import tarfile

from leapp import reporting
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import NtpMigrationDecision

files = [
    '/etc/ntp.conf', '/etc/ntp/keys',
    '/etc/ntp/crypto/pw', '/etc/ntp/step-tickers'
]

related = [
    reporting.RelatedResource('package', 'ntpd'),
    reporting.RelatedResource('package', 'chrony'),
] + [reporting.RelatedResource('file', f) for f in files]


# Check if a service is active and enabled
def check_service(name):
    for state in ['active', 'enabled']:
        try:
            run(['systemctl', 'is-{}'.format(state), name])
            api.current_logger().debug('{} is {}'.format(name, state))
        except CalledProcessError:
            api.current_logger().debug('{} is not {}'.format(name, state))
            return False

    return True


# Check if a file exists
def is_file(name):
    return os.path.isfile(name)


# Get a base64-encoded gzipped tarball of specified files
def get_tgz64(filenames):
    stream = io.BytesIO()
    tar = tarfile.open(fileobj=stream, mode='w:gz')
    for filename in filenames:
        if os.path.isfile(filename):
            tar.add(filename)
    tar.close()

    return base64.b64encode(stream.getvalue())


# Check services from the ntp packages for migration
def check_ntp(installed_packages):
    service_data = [('ntpd', 'ntp', '/etc/ntp.conf'),
                    ('ntpdate', 'ntpdate', '/etc/ntp/step-tickers'),
                    ('ntp-wait', 'ntp-perl', None)]

    migrate_services = []
    migrate_configs = []
    for service, package, main_config in service_data:
        if package in installed_packages and \
                check_service('{}.service'.format(service)) and \
                (not main_config or is_file(main_config)):
            migrate_services.append(service)
            if main_config:
                migrate_configs.append(service)

    if migrate_configs:
        reporting.create_report([
            reporting.Title('{} configuration will be migrated'.format(' and '.join(migrate_configs))),
            reporting.Summary('{} service(s) detected to be enabled and active'.format(', '.join(migrate_services))),
            reporting.Severity(reporting.Severity.LOW),
            reporting.Groups([reporting.Groups.SERVICES, reporting.Groups.TIME_MANAGEMENT]),
        ] + related)

        # Save configuration files that will be renamed in the upgrade
        config_tgz64 = get_tgz64(files)
    else:
        api.current_logger().info('ntpd/ntpdate configuration will not be migrated')
        migrate_services = []
        config_tgz64 = ''

    return NtpMigrationDecision(migrate_services=migrate_services, config_tgz64=config_tgz64)



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class RepositoriesBlacklisted(Model):
    """
    Repositories ID that should be ignored by Leapp during upgrade process
    """
    topic = SystemFactsTopic

    repoids = fields.List(fields.String(), default=[])



from leapp.libraries.actor import private


def test_checkfirewalld_ipset():
    assert private.isIpsetTypeSupportedByNftables('hash:mac')
    assert private.isIpsetTypeSupportedByNftables('hash:ip')
    assert private.isIpsetTypeSupportedByNftables('hash:net')

    assert not private.isIpsetTypeSupportedByNftables('hash:ip,mark')
    assert not private.isIpsetTypeSupportedByNftables('hash:ip,port')
    assert not private.isIpsetTypeSupportedByNftables('hash:ip,port,ip')
    assert not private.isIpsetTypeSupportedByNftables('hash:ip,port,net')
    assert not private.isIpsetTypeSupportedByNftables('hash:net,iface')
    assert not private.isIpsetTypeSupportedByNftables('hash:net,net')
    assert not private.isIpsetTypeSupportedByNftables('hash:net,port')


def test_checkfirewalld_ebtables():
    assert private.isEbtablesTableSupported('nat')
    assert private.isEbtablesTableSupported('filter')

    assert not private.isEbtablesTableSupported('broute')



from leapp import reporting
from leapp.actors import Actor
from leapp.models import GrubConfigError
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


def _create_grub_error_report(error, title, summary, severity=reporting.Severity.LOW,
                              remediation=None, is_inhibitor=False):
    """
    A helper that produces a specific grub error report
    """
    # set default group for a grub error report
    groups = [reporting.Groups.BOOT]
    # set an inhibitor group
    if is_inhibitor:
        groups.append(reporting.Groups.INHIBITOR)
    report_fields = [reporting.Title(title),
                     reporting.Summary(summary),
                     reporting.Severity(severity),
                     reporting.Groups(groups)]
    if remediation:
        report_fields.append(remediation)
    # add information about grub config files
    report_fields.extend([reporting.RelatedResource('file', config_file) for config_file in error.files])
    # finally produce a report
    create_report(report_fields)


class DetectGrubConfigError(Actor):
    """
    Check grub configuration for various errors.

    Currently 3 types of errors are detected:
    - Syntax error in GRUB_CMDLINE_LINUX value;
    - Missing newline at the end of file;
    - Grubenv config file has a 1K size and doesn't end with a line feed.

    There should be only one message of each error type. If for any reason there are more - only the first error of
    each type is reported.
    """

    name = 'detect_grub_config_error'
    consumes = (GrubConfigError,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        # syntax error in GRUB_CMDLINE_LINUX, recoverable
        for error in [err for err in self.consume(GrubConfigError)
                      if err.error_type == GrubConfigError.ERROR_GRUB_CMDLINE_LINUX_SYNTAX]:
            _create_grub_error_report(
                    error=error,
                    title='Syntax error detected in grub configuration',
                    summary=('Syntax error was detected in GRUB_CMDLINE_LINUX value of grub configuration. '
                             'This error is causing booting and other issues. '
                             'Error is automatically fixed by add_upgrade_boot_entry actor.'),
                )
            break
        # missing newline, recoverable
        for error in [err for err in self.consume(GrubConfigError)
                      if err.error_type == GrubConfigError.ERROR_MISSING_NEWLINE]:
            _create_grub_error_report(
                    error=error,
                    title='Detected a missing newline at the end of grub configuration file',
                    summary=('The missing newline in /etc/default/grub causes booting issues when appending '
                             'new entries to this file during the upgrade. Leapp will automatically fix this '
                             'problem by appending the missing newline to the grub configuration file.')
                )
            break
        # corrupted configuration, inhibitor
        for error in [err for err in self.consume(GrubConfigError)
                      if err.error_type == GrubConfigError.ERROR_CORRUPTED_GRUBENV]:
            _create_grub_error_report(
                    error=error,
                    title='Detected a corrupted grubenv file',
                    summary=('The grubenv file must be valid to pass the upgrade correctly: \n'
                             '- an exact size of 1024 bytes is expected \n'
                             '- it cannot end with a newline. \n'
                             'The corruption could be caused by a manual modification of the file which '
                             'is not recommended.'),
                    severity=reporting.Severity.HIGH,
                    is_inhibitor=True,
                    remediation=reporting.Remediation(
                        hint='Delete {} file(s) and regenerate grubenv using the grub2-mkconfig tool'.format(
                            ','.join(error.files))))
            break



from leapp.actors import Actor
from leapp.libraries.actor import multipathconfupdate
from leapp.models import MultipathConfFacts
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class MultipathConfUpdate(Actor):
    """
    Modifies multipath configuration files on the target RHEL-8 system so that
    they will run properly. This is done in three ways
    1. commenting out lines for options that no longer exist, or whose value
       is no longer current in RHEL-8
    2. Migrating any options in an devices section with all_devs to an
       overrides sections
    3. Rename options that have changed names
    """

    name = 'multipath_conf_update'
    consumes = (MultipathConfFacts,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        facts = next(self.consume(MultipathConfFacts), None)
        if facts is None:
            self.log.debug('Skipping execution. No MultipathConfFacts has '
                           'been produced')
            return
        multipathconfupdate.update_configs(facts)



from leapp.models import InstalledDesktopsFacts, InstalledKdeAppsFacts, Report
from leapp.utils.report import is_inhibitor

no_desktop_env = InstalledDesktopsFacts(gnome_installed=False,
                                        kde_installed=False)
gnome_desktop_env = InstalledDesktopsFacts(gnome_installed=True,
                                           kde_installed=False)
KDE_desktop_env = InstalledDesktopsFacts(gnome_installed=False,
                                         kde_installed=True)
both_desktop_env = InstalledDesktopsFacts(gnome_installed=True,
                                          kde_installed=True)


no_KDE_apps = InstalledKdeAppsFacts(installed_apps=[])
some_KDE_apps = InstalledKdeAppsFacts(installed_apps=["okular", "kate"])


def test_no_desktop_no_apps(current_actor_context):
    """
    No action expected.
    """
    current_actor_context.feed(no_desktop_env)
    current_actor_context.feed(no_KDE_apps)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_gnome_desktop_no_apps(current_actor_context):
    """
    No action expected.
    """
    current_actor_context.feed(gnome_desktop_env)
    current_actor_context.feed(no_KDE_apps)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_gnome_desktop_KDE_apps(current_actor_context):
    """
    One report about deleting KDE apps expected.
    """
    current_actor_context.feed(gnome_desktop_env)
    current_actor_context.feed(some_KDE_apps)
    current_actor_context.run()
    message = current_actor_context.consume(Report)[0]
    assert "Upgrade can be performed, but KDE/Qt apps will be uninstalled." in message.report["title"]


def test_KDE_desktop_no_apps(current_actor_context):
    """
    "Inhibitor" flag in report expected.
    """
    current_actor_context.feed(KDE_desktop_env)
    current_actor_context.feed(no_KDE_apps)
    current_actor_context.run()
    message = current_actor_context.consume(Report)[0]
    assert is_inhibitor(message.report)


def test_KDE_desktop_KDE_apps(current_actor_context):
    """
    "Inhibitor" flag in report expected.
    """
    current_actor_context.feed(KDE_desktop_env)
    current_actor_context.feed(some_KDE_apps)
    current_actor_context.run()
    message = current_actor_context.consume(Report)[0]
    assert is_inhibitor(message.report)


def test_both_desktops_no_apps(current_actor_context):
    """
    Report about removing KDE desktop environment expected.
    """
    current_actor_context.feed(both_desktop_env)
    current_actor_context.feed(no_KDE_apps)
    current_actor_context.run()
    message = current_actor_context.consume(Report)[0]
    assert "Upgrade can be performed, but KDE will be uninstalled." in message.report["title"]


def test_both_desktop_KDE_apps(current_actor_context):
    """
    Two reports expected, first about removing KDE desktop, second about KDE/Qt apps
    """
    current_actor_context.feed(both_desktop_env)
    current_actor_context.feed(some_KDE_apps)
    current_actor_context.run()
    messages = current_actor_context.consume(Report)
    remove_KDE_title = "Upgrade can be performed, but KDE will be uninstalled."
    remove_apps_title = "Upgrade can be performed, but KDE/Qt apps will be uninstalled."
    assert len(messages) == 2
    assert [True for message in messages if remove_KDE_title in message.report["title"]]
    assert [True for message in messages if remove_apps_title in message.report["title"]]



import os

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api, run
from leapp.models import CryptoPolicyInfo, CustomCryptoPolicy, CustomCryptoPolicyModule

CRYPTO_CURRENT_STATE_FILE = '/etc/crypto-policies/state/current'
CRYPTO_POLICIES_POLICY_DIRS = ('/etc/crypto-policies/policies',
                               '/usr/share/crypto-policies/policies',)
CRYPTO_POLICIES_MODULES_DIRS = ('/etc/crypto-policies/policies/modules',
                                '/usr/share/crypto-policies/policies/modules',)


def read_current_policy(file):
    if not os.path.exists(file):
        # NOTE(pstodulk) just seatbelt, I do not expect the file is not present
        # skipping tests
        raise StopActorExecutionError(
                'File not found: {}'.format(file),
                details={'details:': 'Cannot check the current set crypto policies.'}
        )
    current = 'DEFAULT'
    with open(file) as fp:
        current = fp.read().strip()
    return current


def _get_name_from_file(file):
    """This is just stripping the path and the extension"""
    base = os.path.basename(file)
    return os.path.splitext(base)[0]


def find_rpm_untracked(files):
    """Check if the list of files is tracked by RPM"""
    if not files:
        return []
    try:
        res = run(['rpm', '-Vf', ] + files, split=True, checked=False)
    except OSError as err:
        error = 'Failed to invoke rpm to check untracked files: {}'.format(str(err))
        api.current_logger().error(error)
        return []

    # return only untracked files from the list
    out = []
    for file in files:
        exp = "file {} is not owned by any package".format(file)
        if exp in res['stdout']:
            out.append(file)
    return out


def read_policy_dirs(dirs, obj, extension):
    """List files with given extension in given directories. Returns only the ones that are not tracked by RPM"""
    files = []
    # find all policy files
    for d in dirs:
        for file in os.listdir(d):
            file = os.path.join(d, file)
            if not os.path.isfile(file) or not file.endswith(extension):
                continue
            files.append(file)
    # now, check which are not tracked by RPM:
    files = find_rpm_untracked(files)
    out = []
    for file in files:
        name = _get_name_from_file(file)
        out.append(obj(name=name, path=file))

    return out


def process():
    current = read_current_policy(CRYPTO_CURRENT_STATE_FILE)

    policies = read_policy_dirs(CRYPTO_POLICIES_POLICY_DIRS, CustomCryptoPolicy, ".pol")
    modules = read_policy_dirs(CRYPTO_POLICIES_MODULES_DIRS, CustomCryptoPolicyModule, ".pmod")

    api.produce(CryptoPolicyInfo(current_policy=current,
                                 custom_policies=policies,
                                 custom_modules=modules))



import filecmp
import os

from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import mounting
from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import CalledProcessError
from leapp.models import TargetUserSpaceInfo
from leapp.tags import IPUWorkflowTag, PreparationPhaseTag


class ZiplConvertToBLSCFG(Actor):
    """
    Convert the zipl boot loader configuration to the the boot loader specification on s390x systems.
    """

    name = 'zipl_convert_to_blscfg'
    consumes = (TargetUserSpaceInfo,)
    produces = ()
    tags = (IPUWorkflowTag, PreparationPhaseTag)

    def process(self):
        if not architecture.matches_architecture(architecture.ARCH_S390X):
            return
        userspace = next(self.consume(TargetUserSpaceInfo), None)
        if not userspace:
            # actually this should not happen, but in such case, we want to still
            # rather continue even if we boot into the old kernel, but in such
            # case, people will have to do manual actions.
            # NOTE: it is really just hypothetical
            self.log_error(
                'TargetUserSpaceInfo is missing. Cannot execute zipl-switch-to-blscfg'
                ' to convert the zipl configuration to BLS.'
            )
            raise StopActorExecutionError('GENERAL FAILURE: Input data for the actor are missing.')

        # replace the original boot directory inside the container by the host one
        # - as we cannot use zipl* pointing anywhere else than default directory
        # - no, --bls-directory is not solution
        # also make sure device nodes are available (requirement for zipl-switch-to-blscfg)
        binds = ['/boot', '/dev']
        with mounting.NspawnActions(base_dir=userspace.path, binds=binds) as context:
            userspace_zipl_conf = os.path.join(userspace.path, 'etc', 'zipl.conf')
            if os.path.exists(userspace_zipl_conf):
                os.remove(userspace_zipl_conf)
            context.copy_to('/etc/zipl.conf', '/etc/zipl.conf')
            # zipl needs this one as well
            context.copy_to('/etc/machine-id', '/etc/machine-id')
            try:
                context.call(['/usr/sbin/zipl-switch-to-blscfg'])
                if filecmp.cmp('/etc/zipl.conf', userspace_zipl_conf):
                    # When the files are same, zipl failed - see the switch script
                    raise OSError('Failed to convert the ZIPL configuration to BLS.')
                context.copy_from('/etc/zipl.conf', '/etc/zipl.conf')
            except OSError as e:
                self.log.error('Could not call zipl-switch-to-blscfg command.',
                               exc_info=True)
                raise StopActorExecutionError(
                    message='Failed to execute zipl-switch-to-blscfg.',
                    details={'details': str(e)}
                )
            except CalledProcessError as e:
                self.log.error('zipl-switch-to-blscfg execution failed,',
                               exc_info=True)
                raise StopActorExecutionError(
                    message='zipl-switch-to-blscfg execution failed with non zero exit code.',
                    details={'details': str(e), 'stdout': e.stdout, 'stderr': e.stderr}
                )

                # FIXME: we do not want to continue anymore, but we should clean
                # better.
                # NOTE: Basically, just removal of the /boot/loader dir content inside
                # could be enough, but we cannot remove /boot/loader because of boom
                # - - if we remove it, we will remove the snapshot as well
                # - - on the other hand, we shouldn't keep it there if zipl
                # - - has not been converted to BLS



from leapp.actors import Actor
from leapp.libraries.actor import vimmigrate
from leapp.models import DistributionSignedRPM
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class VimMigrate(Actor):
    """
    Modify configuration files of Vim 8.0 and later to keep the same behavior
    as Vim 7.4 and earlier had.
    """

    name = 'vim_migrate'
    consumes = (DistributionSignedRPM,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        vimmigrate.update_vim()



"""
Script for installing dependencies for specific actor.
It is called from Makefile install-deps target.
If given actor doesn't exist, script exits with return
code 1 and stderr message.
If given actor doesn't have Makefile, warning printed on stderr.
If no actor is specified dependencies will be installed
for all actors with Makefile.

usage: python install_actor_deps.py [--actor actorname]
"""

import argparse
import os
import sys
from subprocess import check_call, CalledProcessError


def error(msg, rc):
    sys.stderr.write(msg)
    sys.exit(rc)


def install(path):
    cmd = "make -f {} install-deps".format(path)
    try:
        check_call(cmd, shell=True)
    except CalledProcessError as e:
        error(str(e) + '\n', e.returncode)


def install_actor_deps(actor, directory):
    for root, dirs, files in os.walk(directory):
        if actor in dirs:
            makefile_path = os.path.join(root, actor, 'Makefile')
            if os.path.isfile(makefile_path):
                install(makefile_path)
            else:
                sys.stderr.write("Actor '{}' doesn't have Makefile!\n".format(actor))
            return
    error("Actor '{}' doesn't exist!\n".format(actor), 1)


def install_all_deps(directory, repos):
    repos = repos.split() if repos else repos
    for root, dirs, files in os.walk(directory):
        if repos and not any([repo in dirs for repo in repos]):
            continue
        if 'Makefile' in files:
            install(os.path.join(root, 'Makefile'))


if __name__ == "__main__":
    ACTORS_DIR = './repos'

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--actor", help="name of the actor for which to install dependencies")
    parser.add_argument(
        "--repos", help="repositories to look into")
    args = parser.parse_args()

    if args.actor:
        install_actor_deps(args.actor, ACTORS_DIR)
    else:
        install_all_deps(ACTORS_DIR, args.repos)



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class Group(Model):
    topic = SystemFactsTopic

    name = fields.String()
    gid = fields.Integer()
    members = fields.List(fields.String())


class GroupsFacts(Model):
    topic = SystemFactsTopic

    groups = fields.List(fields.Model(Group))



from leapp.actors import Actor
from leapp.libraries.actor import spamassassinconfigupdate
from leapp.models import SpamassassinFacts
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class SpamassassinConfigUpdate(Actor):
    """
    This actor performs several modifications to spamassassin configuration
    so that spamc and the spamassassin systemd service can be run without error
    on the target system:
    1. Remove arguments given to the --ssl option in spamc configuration
       (/etc/mail/spamassassin/spamc.conf).
    2. Remove --ssl-version options from the spamassassin sysconfig file
       (/etc/sysconfig/spamassassin), or replace them with --ssl, if needed.
    3. Remove the -d/--daemonize option from the spamassassin sysconfig file.

    All files are backed up before they are modified.
    """

    name = 'spamassassin_config_update'
    consumes = (SpamassassinFacts,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        facts = next(self.consume(SpamassassinFacts), None)
        if facts:
            spamassassinconfigupdate.migrate_configs(facts)
        else:
            self.log.debug('Skipping execution - no SpamassassinFacts message has been produced.')



from leapp import reporting
from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.config import version
from leapp.models import DracutModule, FIPSInfo, Report, UpgradeInitramfsTasks
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckFips(Actor):
    """
    Inhibit upgrade if FIPS is detected as enabled.
    """

    name = 'check_fips'
    consumes = (FIPSInfo,)
    produces = (Report, UpgradeInitramfsTasks)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        fips_info = next(self.consume(FIPSInfo), None)

        if not fips_info:
            raise StopActorExecutionError('Cannot check FIPS state due to not receiving necessary FIPSInfo message',
                                          details={'Problem': 'Did not receive a message with information about FIPS '
                                                              'usage'})

        if version.get_target_major_version() == '8':
            if fips_info.is_enabled:
                title = 'Automated upgrades from RHEL 7 to RHEL 8 in FIPS mode are not supported'
                summary = ('Leapp has detected that FIPS is enabled on this system. '
                           'Automated in-place upgrade of RHEL 7 systems in FIPS mode is currently unsupported '
                           'and manual intervention is required.')

                fips_7to8_steps_docs_url = 'https://red.ht/planning-upgrade-to-rhel8'

                reporting.create_report([
                    reporting.Title(title),
                    reporting.Summary(summary),
                    reporting.Severity(reporting.Severity.HIGH),
                    reporting.Groups([reporting.Groups.SECURITY, reporting.Groups.INHIBITOR]),
                    reporting.ExternalLink(url=fips_7to8_steps_docs_url,
                                           title='Planning an upgrade from RHEL 7 to RHEL 8')
                ])
        else:
            # FIXME(mhecko): We include these files manually as they are not included automatically when the fips
            # module is used due to a bug in dracut. This code should be removed, once the dracut bug is resolved.
            # See https://bugzilla.redhat.com/show_bug.cgi?id=2176560
            if fips_info.is_enabled:
                fips_required_initramfs_files = [
                    '/etc/crypto-policies/back-ends/opensslcnf.config',
                    '/etc/pki/tls/openssl.cnf',
                    '/usr/lib64/ossl-modules/fips.so',
                ]
                self.produce(UpgradeInitramfsTasks(include_files=fips_required_initramfs_files,
                                                   include_dracut_modules=[DracutModule(name='fips')]))



from leapp.actors import Actor
from leapp.libraries.actor import checkrhsmsku
from leapp.models import Report, RHSMInfo
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckRedHatSubscriptionManagerSKU(Actor):
    """
    Ensure the system is subscribed to the subscription manager

    This actor verifies that the system is correctly subscribed to via the Red Hat Subscription Manager and
    has attached SKUs. The actor will inhibit the upgrade if there are none and RHSM is not supposed
    to be skipped.
    """

    name = 'check_rhsmsku'
    consumes = (RHSMInfo,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        checkrhsmsku.process()



from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM

# The main SSHD configuration file
SSHD_CONFIG = '/etc/ssh/sshd_config'

# The include directive needed, taken from RHEL9 sshd_config with leapp comment
INCLUDE = 'Include /etc/ssh/sshd_config.d/*.conf'
INCLUDE_BLOCK = ''.join(('# Added by leapp during upgrade from RHEL8 to RHEL9\n', INCLUDE, '\n'))


def prepend_string_if_not_present(f, content, check_string):
    """
    This reads the open file descriptor and checks for presence of the `check_string`.
    If not present, the `content` is prepended to the original content of the file and
    result is written.
    Note, that this requires opened file for both reading and writing, for example with:

        with open(path, r+') as f:
    """
    lines = f.readlines()
    for line in lines:
        if line.lstrip().startswith(check_string):
            # The directive is present
            return

    # prepend it otherwise, also with comment
    f.seek(0)
    f.write(''.join((content, ''.join(lines))))


def process(openssh_messages):
    """
    The main logic of the actor:
     * read the configuration file message
     * skip if no action is needed
       * package not installed
       * the configuration file was not modified
     * insert the include directive if it is not present yet
    """
    config = next(openssh_messages, None)
    if list(openssh_messages):
        api.current_logger().warning('Unexpectedly received more than one OpenSshConfig message.')
    if not config:
        raise StopActorExecutionError(
            'Could not check openssh configuration', details={'details': 'No OpenSshConfig facts found.'}
        )

    # If the package is not installed, there is no need to do anything
    if not has_package(DistributionSignedRPM, 'openssh-server'):
        return

    # If the configuration file was not modified, the rpm update will bring the new
    # changes by itself
    if not config.modified:
        return

    # otherwise prepend the Include directive to the main sshd_config
    api.current_logger().debug('Adding the Include directive to {}.'
                               .format(SSHD_CONFIG))
    try:
        with open(SSHD_CONFIG, 'r+') as f:
            prepend_string_if_not_present(f, INCLUDE_BLOCK, INCLUDE)
    except (OSError, IOError) as error:
        api.current_logger().error('Failed to modify the file {}: {} '.format(SSHD_CONFIG, error))



from leapp.libraries.common.rpms import has_package
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM

# Database of changes in configuration files of sane-backends
# between RHELs

CANON_DR = [
    '# P-150M',
    'usb 0x1083 0x162c',
    '# DR-M160',
    'option extra-status 1',
    'option duplex-offset 400',
    'usb 0x1083 0x163e',
    '# DR-M140',
    'option extra-status 1',
    'option duplex-offset 400',
    'usb 0x1083 0x163f',
    '# DR-C125',
    'option duplex-offset 400',
    'usb 0x1083 0x1640',
    '# DR-P215',
    'usb 0x1083 0x1641',
    '# FSU-201',
    'usb 0x1083 0x1648',
    '# DR-C130',
    'usb 0x1083 0x164a',
    '# DR-P208',
    'usb 0x1083 0x164b',
    '# DR-G1130',
    'option buffer-size 8000000',
    'usb 0x1083 0x164f',
    '# DR-G1100',
    'option buffer-size 8000000',
    'usb 0x1083 0x1650',
    '# DR-C120',
    'usb 0x1083 0x1651',
    '# P-201',
    'usb 0x1083 0x1652',
    '# DR-F120',
    'option duplex-offset 1640',
    'usb 0x1083 0x1654',
    '# DR-M1060',
    'usb 0x1083 0x1657',
    '# DR-C225',
    'usb 0x1083 0x1658',
    '# DR-P215II',
    'usb 0x1083 0x1659',
    '# P-215II',
    'usb 0x1083 0x165b',
    '# DR-P208II',
    'usb 0x1083 0x165d',
    '# P-208II',
    'usb 0x1083 0x165f'
]

CARDSCAN = [
    '# Sanford Cardscan 800c',
    'usb 0x0451 0x6250'
]

DLL = ['epsonds']

EPJITSU = [
    '# Fujitsu fi-65F',
    'firmware /usr/share/sane/epjitsu/65f_0A01.nal',
    'usb 0x04c5 0x11bd',
    '# Fujitsu S1100',
    'firmware /usr/share/sane/epjitsu/1100_0B00.nal',
    'usb 0x04c5 0x1200',
    '# Fujitsu S1300i',
    'firmware /usr/share/sane/epjitsu/1300i_0D12.nal',
    'usb 0x04c5 0x128d',
    '# Fujitsu S1100i',
    'firmware /usr/share/sane/epjitsu/1100i_0A00.nal',
    'usb 0x04c5 0x1447'
]

FUJITSU = [
    '#fi-6125',
    'usb 0x04c5 0x11ee',
    '#fi-6225',
    'usb 0x04c5 0x11ef',
    '#ScanSnap SV600',
    'usb 0x04c5 0x128e',
    '#fi-7180',
    'usb 0x04c5 0x132c',
    '#fi-7280',
    'usb 0x04c5 0x132d',
    '#fi-7160',
    'usb 0x04c5 0x132e',
    '#fi-7260',
    'usb 0x04c5 0x132f',
    '#ScanSnap iX500EE',
    'usb 0x04c5 0x13f3',
    '#ScanSnap iX100',
    'usb 0x04c5 0x13f4',
    '#ScanPartner SP25',
    'usb 0x04c5 0x1409',
    '#ScanPartner SP30',
    'usb 0x04c5 0x140a',
    '#ScanPartner SP30F',
    'usb 0x04c5 0x140c',
    '#fi-6140ZLA',
    'usb 0x04c5 0x145f',
    '#fi-6240ZLA',
    'usb 0x04c5 0x1460',
    '#fi-6130ZLA',
    'usb 0x04c5 0x1461',
    '#fi-6230ZLA',
    'usb 0x04c5 0x1462',
    '#fi-6125ZLA',
    'usb 0x04c5 0x1463',
    '#fi-6225ZLA',
    'usb 0x04c5 0x1464',
    '#fi-6135ZLA',
    'usb 0x04c5 0x146b',
    '#fi-6235ZLA',
    'usb 0x04c5 0x146c',
    '#fi-6120ZLA',
    'usb 0x04c5 0x146d',
    '#fi-6220ZLA',
    'usb 0x04c5 0x146e',
    '#N7100',
    'usb 0x04c5 0x146f',
    '#fi-6400',
    'usb 0x04c5 0x14ac',
    '#fi-7480',
    'usb 0x04c5 0x14b8',
    '#fi-6420',
    'usb 0x04c5 0x14bd',
    '#fi-7460',
    'usb 0x04c5 0x14be',
    '#fi-7140',
    'usb 0x04c5 0x14df',
    '#fi-7240',
    'usb 0x04c5 0x14e0',
    '#fi-7135',
    'usb 0x04c5 0x14e1',
    '#fi-7235',
    'usb 0x04c5 0x14e2',
    '#fi-7130',
    'usb 0x04c5 0x14e3',
    '#fi-7230',
    'usb 0x04c5 0x14e4',
    '#fi-7125',
    'usb 0x04c5 0x14e5',
    '#fi-7225',
    'usb 0x04c5 0x14e6',
    '#fi-7120',
    'usb 0x04c5 0x14e7',
    '#fi-7220',
    'usb 0x04c5 0x14e8',
    '#fi-400F',
    'usb 0x04c5 0x151e',
    '#fi-7030',
    'usb 0x04c5 0x151f',
    '#fi-7700',
    'usb 0x04c5 0x1520',
    '#fi-7600',
    'usb 0x04c5 0x1521',
    '#fi-7700S',
    'usb 0x04c5 0x1522'
]

CANON = [
    '# Canon LiDE 80',
    'usb 0x04a9 0x2214',
    '# Canon LiDE 120',
    'usb 0x04a9 0x190e',
    '# Canon LiDE 220',
    'usb 0x04a9 0x190f'
]

XEROX_MFP = [
    '#Samsung X4300 Series',
    'usb 0x04e8 0x3324',
    '#Samsung K4350 Series',
    'usb 0x04e8 0x3325',
    '#Samsung X7600 Series',
    'usb 0x04e8 0x3326',
    '#Samsung K7600 Series',
    'usb 0x04e8 0x3327',
    '#Samsung K703 Series',
    'usb 0x04e8 0x3331',
    '#Samsung X703 Series',
    'usb 0x04e8 0x3332',
    '#Samsung M458x Series',
    'usb 0x04e8 0x346f',
    '#Samsung M4370 5370 Series',
    'usb 0x04e8 0x3471',
    '#Samsung X401 Series',
    'usb 0x04e8 0x3477',
    '#Samsung K401 Series',
    'usb 0x04e8 0x3478',
    '#Samsung K3250 Series',
    'usb 0x04e8 0x3481',
    '#Samsung X3220 Series',
    'usb 0x04e8 0x3482'
]

NEW_QUIRKS = {
    '/etc/sane.d/canon_dr.conf': CANON_DR,
    '/etc/sane.d/cardscan.conf': CARDSCAN,
    '/etc/sane.d/dll.conf': DLL,
    '/etc/sane.d/epjitsu.conf': EPJITSU,
    '/etc/sane.d/fujitsu.conf': FUJITSU,
    '/etc/sane.d/canon.conf': CANON,
    '/etc/sane.d/xerox_mfp.conf': XEROX_MFP
}
"""
Dictionary of configuration files which changes in 1.0.27
"""


def _macro_exists(path, macro):
    """
    Check if macro is in the file.

    :param str path: string representing the full path of the config file
    :param str macro: new directive to be added
    :return boolean res: macro does/does not exist in the file
    """
    with open(path, 'r') as f:
        lines = f.readlines()

    for line in lines:
        if line.lstrip().startswith(macro):
            return True
    return False


def _append_string(path, content):
    """
    Append string at the end of file.

    :param str path: string representing the full path of file
    :param str content: preformatted string to be added
    """
    with open(path, 'a') as f:
        f.write(content)


def update_config(path,
                  quirks,
                  check_function=_macro_exists,
                  append_function=_append_string):
    """
    Insert expected content into the file on the path if it is not
    in the file already.

    :param str path: string representing the full path of the config file
    :param func check_function: function to be used to check if string is in the file
    :param func append_function: function to be used to append string
    """

    macros = []
    for macro in quirks:
        if not check_function(path, macro):
            macros.append(macro)

    if not macros:
        return

    fmt_input = "\n{comment_line}\n{content}\n".format(comment_line='# content added by Leapp',
                                                       content='\n'.join(macros))

    try:
        append_function(path, fmt_input)
    except IOError:
        raise IOError('Error during writing to file: {}.'.format(path))


def _check_package(pkg_name):
    """
    Checks if the package is installed and signed by Red Hat

    :param str pkg_name: name of package
    """

    return has_package(DistributionSignedRPM, pkg_name)


def update_sane(debug_log=api.current_logger().debug,
                error_log=api.current_logger().error,
                is_installed=_check_package,
                append_function=_append_string,
                check_function=_macro_exists):
    """
    Iterate over dictionary and updates each configuration file.

    :param func debug_log: function for debug logging
    :param func error_log: function for error logging
    :param func is_installed: checks if the package is installed
    :param func append_function: appends a string into file
    :param func check_function: checks if a string exists in file
    """

    error_list = []

    if not is_installed('sane-backends'):
        return

    for path, lines in NEW_QUIRKS.items():

        debug_log('Updating SANE configuration file {}.'.format(path))

        try:
            update_config(path, lines, check_function, append_function)
        except (OSError, IOError) as error:
            error_list.append((path, error))

    if error_list:
        error_log('The files below have not been modified '
                  '(error message included):' +
                  ''.join(['\n    - {}: {}'.format(err[0], err[1])
                          for err in error_list]))
        return



import os
import shutil
import tempfile

from leapp.libraries.actor import scanblacklistca
from leapp.libraries.actor.scanblacklistca import _get_files
from leapp.libraries.common.testutils import CurrentActorMocked, produce_mocked
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import BlackListCA, BlackListError

CURDIR = os.path.dirname(os.path.abspath(__file__))
TESTCERT = "badca.cert"
TESTLINK = "linkca.cert"
SUBDIR = "casdir"


class MockedGetFiles(object):
    def __init__(self, files=None, error=None):
        self.called = 0
        self.files = files
        self.error = error
        self.targets = []

    def __call__(self, directory):
        self.targets.append(directory)
        self.called += 1
        if self.error:
            pret = {'signal': 0, 'exit_code': 0xff, 'pid': 0}
            raise CalledProcessError(command="dummy", result=pret, message=self.error)
        ret = []
        for f in self.files:
            ret.append(os.path.join(directory, f))
        return ret


class MockedGetDirs(object):
    def __init__(self, dirs):
        self.called = 0
        self.dirs = dirs

    def __call__(self):
        self.called += 1
        return self.dirs


# make sure get_files is not called if the directory doesn't exist
def test_non_existant_directory(monkeypatch):
    mocked_files = MockedGetFiles()
    monkeypatch.setattr(os.path, 'exists', lambda dummy: False)
    monkeypatch.setattr(scanblacklistca, '_get_files', mocked_files)
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
    monkeypatch.setattr(scanblacklistca, 'run', lambda dummy: dummy)
    scanblacklistca.process()
    assert not mocked_files.called


# unit tests for get_files
def test_get_files(current_actor_context):
    # empty directory
    with tempfile.TemporaryDirectory() as srcdir:
        srcfile = os.path.join(CURDIR, "files", TESTCERT)
        files = _get_files(srcdir)
        assert len(files) == 0
        # single file
        shutil.copy(srcfile, srcdir)
        # make sure we can find certs in the directory
        files = _get_files(srcdir)
        assert len(files) == 1
        assert files[0] == os.path.join(srcdir, TESTCERT)
        # file and symbolic link
        os.symlink(srcfile, os.path.join(srcdir, TESTLINK))
        # make sure we can find certs and links together in the directory
        files = _get_files(srcdir)
        assert len(files) == 2
        assert os.path.join(srcdir, TESTCERT) in files
        assert os.path.join(srcdir, TESTLINK) in files

    # single symbolic link
    with tempfile.TemporaryDirectory() as srcdir:
        os.symlink(srcfile, os.path.join(srcdir, TESTLINK))
        # make sure we can find a solo link in the directory
        files = _get_files(srcdir)
        assert len(files) == 1
        assert files[0] == os.path.join(srcdir, TESTLINK)

    # empty subdirectory
    with tempfile.TemporaryDirectory() as srcdir:
        os.mkdir(os.path.join(srcdir, SUBDIR))
        files = _get_files(srcdir)
        assert len(files) == 0
        # make sure we can find certs in the directory
        shutil.copy(os.path.join(CURDIR, "files", TESTCERT), os.path.join(srcdir, SUBDIR))
        files = _get_files(srcdir)
        assert len(files) == 1
        assert files[0] == os.path.join(srcdir, SUBDIR, TESTCERT)


def test_messages(monkeypatch):
    with tempfile.TemporaryDirectory() as srcdir:
        with tempfile.TemporaryDirectory() as targdir:
            mocked_files = MockedGetFiles(files=[TESTCERT, TESTLINK])
            mocked_dirs = MockedGetDirs(dirs={srcdir: targdir})
            monkeypatch.setattr(scanblacklistca, '_get_files', mocked_files)
            monkeypatch.setattr(scanblacklistca, '_get_dirs', mocked_dirs)
            monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
            monkeypatch.setattr(api, "produce", produce_mocked())
            scanblacklistca.process()
            assert mocked_files.called == 1
            assert len(mocked_files.targets) == 1
            assert mocked_files.targets[0] == srcdir
            assert api.produce.called == 2
            assert len(api.produce.model_instances) == 2
            assert isinstance(api.produce.model_instances[0], BlackListCA)
            assert isinstance(api.produce.model_instances[1], BlackListCA)
            assert api.produce.model_instances[0].sourceDir == srcdir
            assert api.produce.model_instances[0].source == os.path.join(srcdir, TESTCERT)
            assert api.produce.model_instances[0].target == os.path.join(targdir, TESTCERT)
            assert api.produce.model_instances[0].targetDir == targdir
            assert api.produce.model_instances[1].sourceDir == srcdir
            assert api.produce.model_instances[1].source == os.path.join(srcdir, TESTLINK)
            assert api.produce.model_instances[1].target == os.path.join(targdir, TESTLINK)
            assert api.produce.model_instances[1].targetDir == targdir


def test_error(monkeypatch):
    with tempfile.TemporaryDirectory() as srcdir:
        with tempfile.TemporaryDirectory() as targdir:
            error = "get files failed"
            mocked_files = MockedGetFiles(error=error)
            mocked_dirs = MockedGetDirs(dirs={srcdir: targdir})
            monkeypatch.setattr(scanblacklistca, '_get_files', mocked_files)
            monkeypatch.setattr(scanblacklistca, '_get_dirs', mocked_dirs)
            monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
            monkeypatch.setattr(api, "produce", produce_mocked())
            scanblacklistca.process()
            assert mocked_files.called == 1
            assert api.produce.called == 1
            assert len(api.produce.model_instances) == 1
            assert isinstance(api.produce.model_instances[0], BlackListError)
            assert api.produce.model_instances[0].sourceDir == srcdir
            assert api.produce.model_instances[0].targetDir == targdir
            assert api.produce.model_instances[0].error == error



import pytest

from leapp.libraries.actor.cupsscanner import interface_script_check

testdata = (
    ('bla', [], False),
    ('/etc/cups/interfaces', [], False),
    ('/etc/cups/interfaces', ['smth'], True),
)


class MockFilesystem(object):
    def __init__(self, path, files):
        self.path = path
        self.files = files

    def check_path(self, path):
        if self.path == path:
            return True
        return False

    def list_dir(self, path):
        if self.path == path:
            return self.files
        return []


@pytest.mark.parametrize("path,files,expected", testdata)
def test_interface_script_check(path, files, expected):
    filesystem = MockFilesystem(path, files)

    ret = interface_script_check(filesystem.check_path,
                                 filesystem.list_dir)

    assert ret == expected



from leapp import reporting
from leapp.exceptions import StopActorExecutionError
from leapp.libraries.stdlib import api


def process(openssh_messages):
    config = next(openssh_messages, None)
    if list(openssh_messages):
        api.current_logger().warning('Unexpectedly received more than one OpenSshConfig message.')
    if not config:
        raise StopActorExecutionError(
            'Could not check openssh configuration', details={'details': 'No OpenSshConfig facts found.'}
        )

    if config.protocol:
        reporting.create_report([
            reporting.Title('OpenSSH configured with removed configuration Protocol'),
            reporting.Summary(
                'OpenSSH is configured with removed configuration '
                'option Protocol. If this used to be for enabling '
                'SSHv1, this is no longer supported in RHEL 8. '
                'Otherwise this option can be simply removed.'
            ),
            reporting.Severity(reporting.Severity.LOW),
            reporting.Groups([
                    reporting.Groups.AUTHENTICATION,
                    reporting.Groups.SECURITY,
                    reporting.Groups.NETWORK,
                    reporting.Groups.SERVICES
            ]),
        ])



import os

from leapp.libraries.common.dnfplugin import api, apply_workarounds, mounting
from leapp.libraries.common.testutils import CurrentActorMocked
from leapp.models import DNFWorkaround


class ShowMessageCurrentActorMocked(CurrentActorMocked):
    def __init__(self, *args, **kwargs):
        super(ShowMessageCurrentActorMocked, self).__init__(*args, **kwargs)
        self._show_messages = []

    @property
    def show_messages(self):
        return self._show_messages

    def show_message(self, message):
        self._show_messages.append(message)


class MockedNotIsolatedActions(object):
    def __init__(self):
        self.called = 0
        self.args = None

    def call(self, args):
        self.called += 1
        self.args = args
        return {'stdout': ''}

    def __call__(self, *args, **kwargs):
        return self


def _get_tool_path(name):
    for directory in os.getenv('LEAPP_COMMON_TOOLS', '').split(':'):
        full_path = os.path.join(directory, name)
        if os.path.isfile(full_path):
            return full_path
    return None


def test_prepare_yum_config(monkeypatch):
    actions = MockedNotIsolatedActions()
    monkeypatch.setattr(api, 'get_tool_path', _get_tool_path)
    monkeypatch.setattr(mounting, 'NotIsolatedActions', actions)
    display_name = 'Test Action Handle Yum Config'
    actor = ShowMessageCurrentActorMocked(
        msgs=(
            DNFWorkaround(
                display_name=display_name,
                script_path='/your/path/might/vary/handleyumconfig'
            ),
        ),
    )
    monkeypatch.setattr(api, 'current_actor', actor)
    apply_workarounds()
    assert actions.called == 1
    assert os.path.basename(actions.args[-1]) == 'handleyumconfig'
    assert actor.show_messages and len(actor.show_messages) == 1
    assert display_name in actor.show_messages[0]



from leapp.models import FirewalldUsedObjectNames

try:
    from firewall.core.fw import Firewall
except ImportError:
    pass


def is_zone_in_use(conf):
    # This does not account for zone assignments given by NetworkManager.
    if conf.get('interfaces', []) or conf.get('sources', []):
        return True

    return False


def is_zone_in_use_tuple(conf):
    conf_dict = {'interfaces': conf[10],
                 'sources': conf[11]}

    return is_zone_in_use(conf_dict)


def is_policy_in_use(conf, used_zones):
    # A policy is in use if both ingress_zones and egress_zones contain at
    # least one of following: an active zone, 'ANY', 'HOST'.
    for zone in conf.get('ingress_zones', []):
        if zone in ['ANY', 'HOST'] or zone in used_zones:
            break
    else:
        return False
    for zone in conf.get('egress_zones', []):
        if zone in ['ANY', 'HOST'] or zone in used_zones:
            return True
    return False


def get_used_services(conf, isZone):
    used_services = set()

    for service in conf.get('services', []):
        used_services.add(service)

    # Also need to look for 'service name="<service>"' in rich rules. The rule
    # strings are normalized by firewalld. zone keyword is 'rules_str'. policy
    # keyword is 'rich_rules'.
    for rule in conf.get('rules_str', []) if isZone else conf.get('rich_rules', []):
        try:
            search = 'service name="'
            start = rule.index(search) + len(search)
            stop = rule[start:].index('"')
            used_services.add(rule[start:start+stop])
        except ValueError:
            pass

    return used_services


def get_used_services_tuple(conf, isZone):
    if not isZone:
        return set()

    conf_dict = {'services': conf[5],
                 'interfaces': conf[10],
                 'sources': conf[11],
                 'rules_str': conf[12]}

    return get_used_services(conf_dict, isZone)


def read_config():
    try:
        fw = Firewall(offline=True)
    except NameError:
        # import failure missing means firewalld is not installed. Just return
        # the defaults.
        return FirewalldUsedObjectNames()

    # This does not actually start firewalld. It just loads the configuration a
    # la firewall-offline-cmd.
    fw.start()

    # Default zone is always in use. Even without assigned interfaces/sources.
    used_zones = set([fw.get_default_zone()])
    for zone in fw.config.get_zones():
        obj = fw.config.get_zone(zone)
        conf = fw.config.get_zone_config(obj)
        if is_zone_in_use_tuple(conf):
            used_zones.add(zone)

    used_policies = []
    for policy in fw.config.get_policy_objects() if hasattr(fw.config, "get_policy_objects") else []:
        obj = fw.config.get_policy_object(policy)
        conf = fw.config.get_policy_object_config_dict(obj)
        if is_policy_in_use(conf, used_zones):
            used_policies.append(policy)

    used_services = set()
    for zone in fw.config.get_zones():
        obj = fw.config.get_zone(zone)
        conf = fw.config.get_zone_config(obj)
        used_services.update(get_used_services_tuple(conf, True))
    for policy in fw.config.get_policy_objects() if hasattr(fw.config, "get_policy_objects") else []:
        obj = fw.config.get_policy_object(policy)
        conf = fw.config.get_policy_object_config_dict(obj)
        used_services.update(get_used_services(conf, False))

    return FirewalldUsedObjectNames(zones=sorted(used_zones),
                                    policies=sorted(used_policies),
                                    services=sorted(used_services))



from leapp.actors import Actor
from leapp.libraries.actor.verifydialogs import check_dialogs
from leapp.models import DialogModel
from leapp.reporting import Report
from leapp.tags import IPUWorkflowTag, ReportPhaseTag


class VerifyDialogs(Actor):
    """
    Check all dialogs and notify that user needs to make some choices.

    Report messages containing all dialogs with questions that need to be answered will be produced.
    """

    name = 'verify_check_results'
    consumes = (DialogModel,)
    produces = (Report,)
    tags = (ReportPhaseTag, IPUWorkflowTag)

    def process(self):
        check_dialogs(inhibit_if_no_userchoice=self.skip_dialogs)



import os
import re

from leapp.libraries.common.config import architecture, version
from leapp.libraries.stdlib import api
from leapp.utils.deprecation import suppress_deprecation

from leapp.models import (  # isort:skip
    CopyFile,
    RequiredUpgradeInitramPackages,  # deprecated
    UpgradeDracutModule,  # deprecated
    DracutModule,
    TargetUserSpaceUpgradeTasks,
    UpgradeInitramfsTasks
)

_REQUIRED_PACKAGES = [
    'binutils',
    'cifs-utils',
    'device-mapper-multipath',
    'dracut',
    'dracut-config-generic',
    'dracut-config-rescue',
    'dracut-network',
    'dracut-tools',
    'fcoe-utils',
    'hostname',
    'iscsi-initiator-utils',
    'kbd',
    'kernel',
    'kernel-core',
    'kernel-modules',
    'keyutils',
    'kmod',
    'lldpad',
    'lvm2',
    'mdadm',
    'nfs-utils',
    'openssh-clients',
    'plymouth',
    'rpcbind',
    'systemd-container',
    'tar'
]


def _create_initram_networking_tasks():
    # include networking-related dracut modules
    modules_map = {'network-manager': ('network-manager', '/etc/leapp-initram-network-manager'),
                   'scripts': ('network', '/etc/leapp-initram-network-scripts')}
    initram_network_chosen = os.getenv('LEAPP_DEVEL_INITRAM_NETWORK', None)
    if initram_network_chosen in modules_map:
        module, touch_file = modules_map[initram_network_chosen]
        yield UpgradeInitramfsTasks(include_dracut_modules=[DracutModule(name=module)])
        # touch expected file
        yield TargetUserSpaceUpgradeTasks(copy_files=[CopyFile(src='/dev/null', dst=touch_file)])


# The decorator is not effective for generators, it has to be used one level
# above
# @suppress_deprecation(UpgradeDracutModule)
def _create_dracut_modules():
    dracut_base_path = api.get_actor_folder_path('dracut')
    if dracut_base_path:
        dracut_base_path = os.path.abspath(dracut_base_path)
        for module in os.listdir(dracut_base_path):
            yield UpgradeDracutModule(
                name=re.sub(r'^\d+', '', module),
                module_path=os.path.join(dracut_base_path, module)
            )
            # NOTE: when the UpgradeDracutModule is dropped, this could be
            # handled just by one msg instead of two
            dm = DracutModule(
                name=re.sub(r'^\d+', '', module),
                module_path=os.path.join(dracut_base_path, module))
            yield UpgradeInitramfsTasks(include_dracut_modules=[dm])


@suppress_deprecation(RequiredUpgradeInitramPackages)
def _create_initram_packages():
    # copy the list as we do not want to affect the constant because of tests
    required_pkgs = _REQUIRED_PACKAGES[:]
    if architecture.matches_architecture(architecture.ARCH_X86_64):
        required_pkgs.append('biosdevname')
    if os.getenv('LEAPP_DEVEL_INITRAM_NETWORK', None) == 'network-manager':
        required_pkgs.append('NetworkManager')
    if version.get_target_major_version() == '9':
        required_pkgs += ['policycoreutils', 'rng-tools']
    return (
        RequiredUpgradeInitramPackages(packages=required_pkgs),
        TargetUserSpaceUpgradeTasks(install_rpms=required_pkgs)
    )


def process():
    api.produce(*tuple(_create_dracut_modules()))
    api.produce(*_create_initram_packages())
    api.produce(*tuple(_create_initram_networking_tasks()))



from leapp import reporting
from leapp.libraries.common.config import architecture, version
from leapp.libraries.stdlib import api
from leapp.models import SapHanaInfo

# SAP HANA Compatibility
# Supported architectures for upgrades with SAP HANA to RHEL 'X'
SAP_HANA_SUPPORTER_ARCHS = {
    '8': [architecture.ARCH_X86_64],
    '9': [architecture.ARCH_X86_64, architecture.ARCH_PPC64LE]
}

SAP_HANA_MINIMAL_MAJOR_VERSION = 2

# RHEL 8.6 target requirements
SAP_HANA_RHEL86_REQUIRED_PATCH_LEVELS = ((5, 59, 2),)
SAP_HANA_RHEL86_MINIMAL_VERSION_STRING = 'HANA 2.0 SPS05 rev 59.02 or later'

# RHEL 9.0 target requirements
SAP_HANA_RHEL90_REQUIRED_PATCH_LEVELS = ((5, 59, 4), (6, 63, 0))
SAP_HANA_RHEL90_MINIMAL_VERSION_STRING = 'HANA 2.0 SPS05 rev 59.04 or later, or SPS06 rev 63 or later'


def _report_skip_check():
    summary = (
        'For the target RHEL releases >=8.8 and >=9.2 '
        'the leapp utility does not check RHEL and SAP HANA 2.0 '
        'versions compatibility. Please ensure your SAP HANA 2.0 '
        'is supported on the target RHEL release and '
        'proceed on your discretion. '
        'SAP HANA: Supported Operating Systems '
        'https://launchpad.support.sap.com/#/notes/2235581')
    remedy_hint = 'Ensure your SAP HANA 2.0 is supported on the target release.'
    reporting.create_report([
        reporting.Title('SAP HANA 2.0 version should be checked prior the upgrade'),
        reporting.Summary(summary),
        reporting.Severity(reporting.Severity.MEDIUM),
        reporting.Groups([reporting.Groups.SANITY]),
        reporting.Remediation(hint=remedy_hint),
        reporting.ExternalLink(url='https://launchpad.support.sap.com/#/notes/2235581',
                               title='SAP HANA: Supported Operating Systems'),
        ])


def _manifest_get(manifest, key, default_value=None):
    for entry in manifest:
        if entry.key == key:
            return entry.value
    return default_value


def running_check(info):
    """ Creates a report if a running instance of SAP HANA has been detected """
    if info.running:
        reporting.create_report([
            reporting.Title('Found running SAP HANA instances'),
            reporting.Summary(
                'In order to perform a system upgrade it is necessary that all instances of SAP HANA are stopped.'
            ),
            reporting.RemediationHint('Shutdown all SAP HANA instances before you continue with the upgrade.'),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.SANITY]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
        ])


def _add_hana_details(target, instance):
    """ Adds instance information into the target dictionary for creating later reports. """
    target.setdefault(instance.name, {'numbers': set(), 'path': instance.path, 'admin': instance.admin})
    target[instance.name]['numbers'].add(instance.instance_number)


def _create_detected_instances_list(details):
    """ Generates report data for detected instances in list form with details """
    result = []
    for name, meta in details.items():
        result.append(('Name: {name}\n'
                       '  Instances: {instances}\n'
                       '  Admin: {admin}\n'
                       '  Path: {path}').format(name=name,
                                                instances=', '.join(meta['numbers']),
                                                admin=meta['admin'],
                                                path=meta['path']))
    if result:
        return '- {}'.format('\n- '.join(result))
    return ''


def _min_ver_string():
    if version.matches_target_version('8.6'):
        ver_str = SAP_HANA_RHEL86_MINIMAL_VERSION_STRING
    else:
        ver_str = SAP_HANA_RHEL90_MINIMAL_VERSION_STRING
    return ver_str


def version1_check(info):
    """ Creates a report for SAP HANA instances running on version 1 """
    found = {}
    for instance in info.instances:
        if _manifest_get(instance.manifest, 'release') == '1.00':
            _add_hana_details(found, instance)

    if found:
        detected = _create_detected_instances_list(found)
        reporting.create_report([
            reporting.Title('Found SAP HANA 1.0 which is not supported with the target version of RHEL'),
            reporting.Summary(
                ('SAP HANA 1.00 is not supported with the version of RHEL you are upgrading to.\n\n'
                 'The following instances have been detected to be version 1.00:\n'
                 '{}'.format(detected))
            ),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.RemediationHint((
                'In order to upgrade RHEL, you will have to upgrade your SAP HANA 1.0 software to '
                'SAP HANA 2.0 supported on the target RHEL release first.')),
            reporting.ExternalLink(url='https://launchpad.support.sap.com/#/notes/2235581',
                                   title='SAP HANA: Supported Operating Systems'),
            reporting.Groups([reporting.Groups.SANITY]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
        ])


def _major_version_check(instance):
    """ Performs the check for the major version of SAP HANA """
    release = _manifest_get(instance.manifest, 'release', '0.00')
    parts = release.split('.')

    try:
        if int(parts[0]) != SAP_HANA_MINIMAL_MAJOR_VERSION:
            api.current_logger().info('Unsupported major version {} for instance {}'.format(release, instance.name))
            return False
        return True
    except (ValueError, IndexError):
        api.current_logger().warning(
            'Failed to parse manifest release field for instance {}'.format(instance.name), exc_info=True)
        return False


def _sp_rev_patchlevel_check(instance, patchlevels):
    """ Checks whether this SP, REV & PatchLevel are eligible """
    number = _manifest_get(instance.manifest, 'rev-number', '000')
    if len(number) > 2 and number.isdigit():
        required_sp_levels = [r[0] for r in patchlevels]
        lowest_sp = min(required_sp_levels)
        highest_sp = max(required_sp_levels)
        sp = int(number[0:2].lstrip('0') or '0')
        if sp < lowest_sp:
            # Less than minimal required SP
            return False
        if sp > highest_sp:
            # Less than minimal required SP
            return True
        for requirements in patchlevels:
            req_sp, req_rev, req_pl = requirements
            if sp == req_sp:
                rev = int(number.lstrip('0') or '0')
                if rev < req_rev:
                    continue
                if rev == req_rev:
                    patch_level = int(_manifest_get(instance.manifest, 'rev-patchlevel', '00').lstrip('0') or '0')
                    if patch_level < req_pl:
                        continue
                return True
        return False
    # if not 'len(number) > 2 and number.isdigit()'
    api.current_logger().warning(
        'Invalid rev-number field value `{}` in manifest for instance {}'.format(number, instance.name))
    return False


def _fullfills_hana_min_version(instance):
    """ Performs a check whether the version of SAP HANA fulfills the minimal requirements for the target RHEL """
    if version.matches_target_version('8.6'):
        patchlevels = SAP_HANA_RHEL86_REQUIRED_PATCH_LEVELS
    else:
        patchlevels = SAP_HANA_RHEL90_REQUIRED_PATCH_LEVELS
    return _major_version_check(instance) and _sp_rev_patchlevel_check(instance, patchlevels)


def version2_check(info):
    """ Performs all checks for SAP HANA 2 and creates a report if anything unsupported has been detected """
    found = {}
    for instance in info.instances:
        if _manifest_get(instance.manifest, 'release', None) == '1.00':
            continue
        if version.matches_target_version('> 8.6', '< 9.0') or version.matches_target_version('> 9.0'):
            # if a target release is >=8.8 or >=9.2, the SAP HANA and RHEL versions compatibility is not checked
            _report_skip_check()
            return
        # if a starget release is 8.6 or 9.0 we still check SAP HANA and RHEL versions compatibility
        if not _fullfills_hana_min_version(instance):
            _add_hana_details(found, instance)

    if found:
        min_ver_string = _min_ver_string()
        detected = _create_detected_instances_list(found)
        reporting.create_report([
            reporting.Title('SAP HANA needs to be updated before the RHEL upgrade'),
            reporting.Summary(
                ('A newer version of SAP HANA is required in order continue with the upgrade.'
                 ' {min_hana_version} is required for the target version of RHEL.\n\n'
                 'The following SAP HANA instances have been detected to be installed with a lower version'
                 ' than required on the target system:\n'
                 '{detected}').format(detected=detected, min_hana_version=min_ver_string)
            ),
            reporting.RemediationHint('Update SAP HANA at least to {}'.format(min_ver_string)),
            reporting.ExternalLink(url='https://launchpad.support.sap.com/#/notes/2235581',
                                   title='SAP HANA: Supported Operating Systems'),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.SANITY]),
            reporting.Groups([reporting.Groups.INHIBITOR]),
        ])


def platform_check():
    """
    Inhibit the upgrade and return False if SAP HANA is running on an unsupported
    architecture for the upgrade.

    Supported architectures:
    - IPU 7 -> 8: x86_64
    - IPU 8 -> 9: x86_64, ppc64le

    In case of the upgrade to a RHEL X version that is not supported for the
    IPU yet, return False and do not report anything, as the upgrade to
    an unsupported version is handled in general in another actor.
    """
    target_major_version = version.get_target_major_version()
    arch = api.current_actor().configuration.architecture

    if target_major_version not in SAP_HANA_SUPPORTER_ARCHS:
        # Do nothing, the inhibitor will be raised by a different actor, but log it
        api.current_logger().error('Upgrade with SAP HANA is not supported to the target OS.')
        return False

    if arch in SAP_HANA_SUPPORTER_ARCHS[target_major_version]:
        return True

    EXTERNAL_LINK = {
        '8': reporting.ExternalLink(
            url='https://access.redhat.com/solutions/5154031',
            title='How to in-place upgrade SAP environments from RHEL 7 to RHEL 8'),
        '9': reporting.ExternalLink(
            url='https://red.ht/how-to-in-place-upgrade-sap-environments-from-rhel-8-to-rhel-9',
            title='How to in-place upgrade SAP environments from RHEL 8 to RHEL 9')
    }

    reporting.create_report([
        reporting.Title('The current architecture is not supported for SAP HANA on the target system'),
        reporting.Summary(
            'The {arch} architecture is not supported for the in-place upgrade'
            ' to the RHEL {version} system with SAP HANA.'
            ' The in-place upgrade with SAP HANA is now supported for the following'
            ' architectures: {supp_archs}.'
            ' For more information please consult the documentation.'
            .format(
                arch=arch,
                supp_archs=', '.join(SAP_HANA_SUPPORTER_ARCHS[target_major_version]),
                version=target_major_version
            )
        ),
        reporting.Severity(reporting.Severity.HIGH),
        reporting.Groups([reporting.Groups.SANITY]),
        reporting.Groups([reporting.Groups.INHIBITOR]),
        reporting.Audience('sysadmin'),
        EXTERNAL_LINK[target_major_version],
    ])
    return False


def perform_check():
    """ Performs all checks for SAP HANA and will skip if the upgrade flavour is not `saphana` """

    if api.current_actor().configuration.flavour != 'saphana':
        # Do not run on non saphana upgrades
        return

    if not platform_check():
        # If this architecture is not supported, there's no sense in continuing.
        return

    info = next(api.consume(SapHanaInfo), None)
    if not info:
        return

    running_check(info)
    version1_check(info)
    version2_check(info)



from leapp.models import fields, Model
from leapp.topics import TransactionTopic


class SkippedRepositories(Model):
    """
    Message that contains all skipped repositories and the packages that will not be upgraded as a result of those
    repositories being skipped.
    """
    topic = TransactionTopic
    repos = fields.List(fields.String(), default=[])
    """ List of repositories ids that are going to be skipped for the upgrade """
    packages = fields.List(fields.String(), default=[])
    """ List of packages that are not going to be upgraded because of skipped repositories """



from leapp.actors import Actor
from leapp.libraries.actor import updateetcsysconfigkernel
from leapp.tags import IPUWorkflowTag, PreparationPhaseTag


class UpdateEtcSysconfigKernel(Actor):
    """
    Update /etc/sysconfig/kernel file.

    In order to proceed with Upgrade process, DEFAULTKERNEL entry should be updated from kernel to
    kernel-core.
    """

    name = 'update_etc_sysconfig_kernel'
    consumes = ()
    produces = ()
    tags = (PreparationPhaseTag, IPUWorkflowTag)

    def process(self):
        updateetcsysconfigkernel.update_kernel_config('/etc/sysconfig/kernel')



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class MultipathConfig8to9(Model):
    """Model representing information about a multipath configuration file"""
    topic = SystemInfoTopic

    pathname = fields.String()
    """Config file path name"""

    config_dir = fields.Nullable(fields.String())
    """Value of config_dir in the defaults section. None if not set"""

    enable_foreign_exists = fields.Boolean(default=False)
    """True if enable_foreign is set in the defaults section"""

    invalid_regexes_exist = fields.Boolean(default=False)
    """True if any regular expressions have the value of "*" """

    allow_usb_exists = fields.Boolean(default=False)
    """True if allow_usb_devices is set in the defaults section."""


class MultipathConfFacts8to9(Model):
    """Model representing information from multipath configuration files"""
    topic = SystemInfoTopic

    configs = fields.List(fields.Model(MultipathConfig8to9), default=[])
    """List of multipath configuration files"""



import pytest

from leapp.libraries.actor.cupsscanner import digest_directive_check

testdata = (
    ('\n', False),
    ('test\n', False),
    ('AuthType Basic\n', False),
    ('DefaultAuthType Basic\n', False),
    ('#AuthType Digest\n', False),
    ('#DefaultAuthType BasicDigest\n', False),
    ('DefaultAuthType BasicDigest\n', True),
    ('DefaultAuthType Digest\n', True),
    ('AuthType Digest\n', True),
    ('AuthType BasicDigest\n', True),
    ('AuthType BasicDigest\nDefaultAuthType Digest\n', True),
)


class MockConfig(object):
    def __init__(self, content):
        self.content = content

    def read(self, path):
        return self.content.splitlines(True)


@pytest.mark.parametrize("content,expected", testdata)
def test_digest_directive_check(content, expected):
    config = MockConfig(content)

    ret = digest_directive_check('does_not_matter', config.read)

    assert ret == expected



from leapp import reporting
from leapp.actors import Actor
from leapp.models import SSSDConfig8to9
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag

COMMON_REPORT_TAGS = [reporting.Groups.AUTHENTICATION, reporting.Groups.SECURITY]

related = [
    reporting.RelatedResource('package', 'sssd'),
    reporting.RelatedResource('file', '/etc/sssd/sssd.conf')
]


class SSSDCheck8to9(Actor):
    """
    Check SSSD configuration for changes in RHEL9 and report them in model.

    Implicit files domain is disabled by default. This may affect local
    smartcard authentication if there is not explicit files domain created.

    If there is no files domain and smartcard authentication is enabled,
    we will notify the administrator.
    """

    name = 'sssd_check_8to9'
    consumes = (SSSDConfig8to9,)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        model = next(self.consume(SSSDConfig8to9), None)
        if not model:
            return

        # enable_files_domain is set explicitly, change of default has no effect
        if model.enable_files_domain_set:
            return

        # there is explicit files domain, implicit files domain has no effect
        if model.explicit_files_domain:
            return

        # smartcard authentication is disabled, implicit files domain has no effect
        if not model.pam_cert_auth:
            return

        create_report([
            reporting.Title('SSSD implicit files domain is now disabled by default.'),
            reporting.Summary('Default value of [sssd]/enable_files_domain has '
                              'changed from true to false.'),
            reporting.Groups(COMMON_REPORT_TAGS),
            reporting.Remediation(
                hint='If you use smartcard authentication for local users, '
                     'set this option to true explicitly and call '
                     '"authselect enable-feature with-files-domain".'
            ),
            reporting.Severity(reporting.Severity.MEDIUM)
        ] + related)



from leapp import reporting
from leapp.actors import Actor
from leapp.models import ActiveKernelModulesFacts
from leapp.reporting import create_report, Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckNvidiaProprietaryDriver(Actor):
    """
    Check if NVIDIA proprietary driver is in use. If yes, inhibit the upgrade process.

    Updating bare metal (or VM) with the binary NVIDIA driver will end up with a blacklisted nouveau.

    See also https://bugzilla.redhat.com/show_bug.cgi?id=2057026
    """

    name = 'check_nvidia_proprietary_driver'
    consumes = (ActiveKernelModulesFacts,)
    produces = (Report,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):

        for fact in self.consume(ActiveKernelModulesFacts):
            nvidia_driver_loaded = any('nvidia' in active_mod.filename for active_mod in fact.kernel_modules)
            if nvidia_driver_loaded:
                create_report([
                    reporting.Title('Proprietary NVIDIA driver detected'),
                    reporting.Summary(
                            'Leapp has detected that the NVIDIA proprietary driver has been loaded, which also means '
                            'the nouveau driver is blacklisted. If you upgrade now, you will end up without a '
                            'graphical session, as the newer kernel won\'t be able to load the NVIDIA driver module '
                            'and nouveau will still be blacklisted.'
                            '\n\n'
                            'Please uninstall the NVIDIA graphics driver before upgrading to make sure you have a '
                            'graphical session after upgrading.'
                    ),
                    reporting.ExternalLink(
                            title='How to uninstall proprietary NVIDIA graphics driver and switch back to Red Hat '
                                  'shipped nouveau graphics driver?',
                            url='https://access.redhat.com/solutions/421683'
                    ),
                    reporting.Severity(reporting.Severity.HIGH),
                    reporting.Groups([reporting.Groups.INHIBITOR]),
                    reporting.Groups([reporting.Groups.KERNEL, reporting.Groups.DRIVERS]),
                ])
                break



from leapp.actors import Actor
from leapp.libraries.actor import cryptopoliciescheck
from leapp.models import CryptoPolicyInfo, Report, TargetUserSpacePreupgradeTasks
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CryptoPoliciesCheck(Actor):
    """
    This actor consumes previously gathered information about crypto policies on the source
    system and does two things:

     * warns user if the custom/legacy policy is used and whether there is time to review it
     * prepares the container by making sure it will have installed the tools for managing
       crypto policies and the custom policies are copied over to the intermediate and target
       systems
    """

    name = 'crypto_policies_check'
    consumes = (CryptoPolicyInfo,)
    produces = (TargetUserSpacePreupgradeTasks, Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag,)

    def process(self):
        cryptopoliciescheck.process(self.consume(CryptoPolicyInfo))



import os.path
from collections import namedtuple

import pytest

from leapp.libraries import stdlib
from leapp.libraries.actor import forcedefaultboot
from leapp.libraries.common.config import architecture
from leapp.libraries.common.testutils import CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import InstalledTargetKernelInfo

Expected = namedtuple(
    'Expected', (
     'grubby_setdefault',
     'zipl_called'
    )
)

Case = namedtuple(
    'Case',
    ('kernel_exists',
     'entry_default',
     'entry_exists',
     'message_available',
     'arch_s390x'
     )
)

TARGET_KERNEL_NEVRA = 'kernel-core-1.2.3.4.el8.x86_64'
TARGET_KERNEL_VERSION = '1.2.3.4.el8.x86_64'
TARGET_KERNEL_TITLE = 'Red Hat Enterprise Linux ({}) 8.1 (Ootpa)'.format(TARGET_KERNEL_VERSION)
TARGET_KERNEL_PATH = '/boot/vmlinuz-{}'.format(TARGET_KERNEL_VERSION)
TARGET_INITRD_PATH = '/boot/initramfs-{}.img'.format(TARGET_KERNEL_VERSION)

OLD_KERNEL_VERSION = '0.1.2.3.el7.x86_64'
OLD_KERNEL_TITLE = 'Red Hat Enterprise Linux ({}) 7.6 (Maipo)'.format(OLD_KERNEL_VERSION)
OLD_KERNEL_PATH = '/boot/vmlinuz-{}'.format(OLD_KERNEL_VERSION)


CASES = (
    (Case(kernel_exists=True, entry_default=True, entry_exists=True, message_available=True, arch_s390x=False),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=False, entry_default=False, entry_exists=True, message_available=True, arch_s390x=False),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=False, entry_exists=True, message_available=False, arch_s390x=False),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=False, entry_exists=False, message_available=False, arch_s390x=False),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=False, entry_exists=True, message_available=True, arch_s390x=False),
     Expected(grubby_setdefault=True, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=True, entry_exists=True, message_available=True, arch_s390x=True),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=False, entry_default=False, entry_exists=True, message_available=True, arch_s390x=True),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=False, entry_exists=True, message_available=False, arch_s390x=True),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=False, entry_exists=False, message_available=False, arch_s390x=True),
     Expected(grubby_setdefault=False, zipl_called=False)),
    (Case(kernel_exists=True, entry_default=False, entry_exists=True, message_available=True, arch_s390x=True),
     Expected(grubby_setdefault=True, zipl_called=True))
)

_GRUBBY_INFO_TEMPLATE = '''index={entry_index}
kernel=/boot/vmlinuz-{kernel_version}
args="ro rd.lvm.lv=testing/root rd.lvm.lv=testing/swap rhgb quiet LANG=en_US.UTF-8"
root=/dev/mapper/testing-root
initrd=/boot/initramfs-{kernel_version}.img
'''


class MockedRun(object):
    def __init__(self, case):
        self.case = case
        self.called_setdefault = False
        self.called_zipl = False

    def __call__(self, cmd, *args, **kwargs):
        if cmd and cmd[0] == 'grubby':
            target = getattr(self, 'grubby_{}'.format(cmd[1].strip('--').replace('-', '_')), None)
            assert target and 'Unsupport grubby command called'  # pylint: disable=simplifiable-condition
            return target(cmd)  # pylint: disable=not-callable
        if cmd and cmd[0] == '/usr/sbin/zipl':
            self.called_zipl = True
        return None

    def grubby_info(self, cmd):
        assert len(cmd) == 3
        if not self.case.entry_exists:
            raise stdlib.CalledProcessError('A leapp command failed', cmd, {})

        return {
            'stdout': _GRUBBY_INFO_TEMPLATE.format(
                entry_index=0 if self.case.entry_default else 1,
                kernel_version=TARGET_KERNEL_VERSION)
            }

    def grubby_default_kernel(self, cmd):
        assert len(cmd) == 2
        if self.case.entry_default:
            return {'stdout': '{}\n'.format(TARGET_KERNEL_PATH)}
        return {'stdout': '{}\n'.format(OLD_KERNEL_PATH)}

    def grubby_default_index(self, cmd):
        assert len(cmd) == 2
        if self.case.entry_default:
            return {'stdout': '1\n'}
        return {'stdout': '0\n'}

    def grubby_default_title(self, cmd):
        assert len(cmd) == 2
        if self.case.entry_default:
            return {'stdout': '{}\n'.format(TARGET_KERNEL_TITLE)}
        return {'stdout': '{}\n'.format(OLD_KERNEL_TITLE)}

    def grubby_set_default(self, cmd):
        assert len(cmd) == 3
        assert cmd[2] == TARGET_KERNEL_PATH
        self.called_setdefault = True


def mocked_consume(case):
    def impl(*args):
        if case.message_available:
            kernel_img_path = TARGET_KERNEL_PATH if case.kernel_exists else ''
            msg = InstalledTargetKernelInfo(pkg_nevra=TARGET_KERNEL_NEVRA,
                                            kernel_img_path=kernel_img_path,
                                            uname_r='',
                                            initramfs_path=TARGET_INITRD_PATH)
            return iter((msg,))
        return iter(())
    return impl


def mocked_exists(case, orig_path_exists):
    def impl(path):
        if path == TARGET_KERNEL_PATH:
            return case.kernel_exists
        if path == TARGET_INITRD_PATH:
            return case.initrd_exists
        return orig_path_exists(path)
    return impl


@pytest.mark.parametrize('case_result', CASES)
def test_force_default_boot_target_scenario(case_result, monkeypatch):
    case, result = case_result
    arch = architecture.ARCH_S390X if case.arch_s390x else architecture.ARCH_X86_64
    mocked_run = MockedRun(case)
    monkeypatch.setattr(api, 'consume', mocked_consume(case))
    monkeypatch.setattr(stdlib, 'run', mocked_run)
    monkeypatch.setattr(os.path, 'exists', mocked_exists(case, os.path.exists))
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=arch))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())
    forcedefaultboot.process()
    assert result.grubby_setdefault == mocked_run.called_setdefault
    assert result.zipl_called == mocked_run.called_zipl



from leapp.actors import Actor
from leapp.libraries.actor import cephvolumescan
from leapp.models import CephInfo, InstalledRPM
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CephVolumeScan(Actor):

    """
    Retrieves the list of encrypted Ceph OSD
    """

    name = 'cephvolumescan'
    consumes = (InstalledRPM,)
    produces = (CephInfo,)
    tags = (ChecksPhaseTag, IPUWorkflowTag)

    def process(self):
        output = cephvolumescan.encrypted_osds_list()
        self.produce(CephInfo(encrypted_volumes=output))



from leapp.actors import Actor
from leapp.libraries.actor import opensshdropindirectory
from leapp.models import DistributionSignedRPM, OpenSshConfig
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class OpenSshDropInDirectory(Actor):
    """
    The RHEL 9 provides default configuration file with an Include directive.

    If the configuration file was modified, it will not be replaced by the update
    and we need to do couple of tweaks:

     * Insert Include directive as expected by the rest of the OS
     * Verify the resulting configuration is valid
       * The only potentially problematic option is "Subsystem", but it is kept in the
         main sshd_config even in RHEL9 so there is no obvious upgrade path where it
         could cause issues (unlike the Debian version).

    [1] https://bugzilla.mindrot.org/show_bug.cgi?id=3236
    """

    name = 'open_ssh_drop_in_directory'
    consumes = (OpenSshConfig, DistributionSignedRPM,)
    produces = ()
    tags = (IPUWorkflowTag, ApplicationsPhaseTag,)

    def process(self):
        opensshdropindirectory.process(self.consume(OpenSshConfig))
